from pathlib import Path

import pytest

from checkov.ansible.checks.registry import registry
from checkov.ansible.runner import Runner
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.severities import Severities, BcSeverities
from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.graph.db_connectors.rustworkx.rustworkx_db_connector import RustworkxConnector
from checkov.runner_filter import RunnerFilter

EXAMPLES_DIR = Path(__file__).parent / "examples"


def test_registry_has_type():
    assert registry.report_type == CheckType.ANSIBLE


@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector,
    ],
)
def test_runner_honors_enforcement_rules(graph_connector):
    # given
    test_file = EXAMPLES_DIR / "site.yml"

    # when
    filter = RunnerFilter(framework=["ansible"], use_enforcement_rules=True)
    # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
    # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
    filter.enforcement_rule_configs = {CheckType.ANSIBLE: Severities[BcSeverities.OFF]}
    report = Runner(db_connector=graph_connector()).run(root_folder="", files=[str(test_file)], runner_filter=filter)

    # then
    summary = report.get_summary()

    assert summary["passed"] == 0
    assert summary["failed"] == 0
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0


@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector,
    ],
)
def test_runner_passing_check(graph_connector):
    # given
    test_file = EXAMPLES_DIR / "site.yml"

    # when
    report = Runner(db_connector=graph_connector()).run(
        root_folder="", files=[str(test_file)], runner_filter=RunnerFilter(checks=["CKV_AWS_135"])
    )

    # then
    summary = report.get_summary()

    assert summary["passed"] == 1
    assert summary["failed"] == 0
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0


@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector,
    ],
)
def test_runner_failing_check(graph_connector):
    # given
    test_file = EXAMPLES_DIR / "site.yml"

    # when
    report = Runner(db_connector=graph_connector()).run(
        root_folder="", files=[str(test_file)], runner_filter=RunnerFilter(checks=["CKV_AWS_88"])
    )

    # then
    summary = report.get_summary()

    assert summary["passed"] == 0
    assert summary["failed"] == 1
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0


@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector,
    ],
)
def test_runner_skipping_check(graph_connector):
    # given
    test_file = EXAMPLES_DIR / "skip.yml"

    # when
    report = Runner(db_connector=graph_connector()).run(
        root_folder="",
        files=[str(test_file)],
        runner_filter=RunnerFilter(checks=["CKV2_ANSIBLE_1", "CKV_AWS_88", "CKV_AWS_135"]),
    )

    # then
    summary = report.get_summary()

    assert summary["passed"] == 0
    assert summary["failed"] == 2
    assert summary["skipped"] == 3
    assert summary["parsing_errors"] == 0

    assert {check.check_id for check in report.skipped_checks} == {"CKV2_ANSIBLE_1", "CKV_AWS_88", "CKV_AWS_135"}

    ansible_1 = next(check for check in report.skipped_checks if check.check_id == "CKV2_ANSIBLE_1")
    aws_88 = next(check for check in report.skipped_checks if check.check_id == "CKV_AWS_88")
    aws_135 = next(check for check in report.skipped_checks if check.check_id == "CKV_AWS_135")
    assert ansible_1.resource == "tasks.uri.http"
    assert aws_88.resource == "tasks.amazon.aws.ec2_instance.Launch ec2 instances 2"
    assert aws_135.resource == "tasks.amazon.aws.ec2_instance.Launch ec2 instances 1"


@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector,
    ],
)
def test_runner_with_flat_tasks(graph_connector):
    # given
    test_file = EXAMPLES_DIR / "tasks.yml"

    # when
    report = Runner(db_connector=graph_connector()).run(
        root_folder="", files=[str(test_file)], runner_filter=RunnerFilter(checks=["CKV_ANSIBLE_1", "CKV_ANSIBLE_2"])
    )

    # then
    summary = report.get_summary()

    assert summary["passed"] == 1
    assert summary["failed"] == 1
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0

    passing_resources = {
        f"tasks.uri.Check that you can connect (GET) to a page",
    }

    failing_resources = {
        f"tasks.ansible.builtin.get_url.Download foo.conf",
    }

    passed_check_resources = {check.resource for check in report.passed_checks}
    failed_check_resources = {check.resource for check in report.failed_checks}

    assert passing_resources == passed_check_resources
    assert failing_resources == failed_check_resources


@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector,
    ],
)
def test_runner_with_block(graph_connector):
    # given
    test_file = EXAMPLES_DIR / "blocks.yml"
    checks = ["CKV_ANSIBLE_3", "CKV_ANSIBLE_4", "CKV2_ANSIBLE_3"]

    # when
    report = Runner(db_connector=graph_connector()).run(
        root_folder="", files=[str(test_file)], runner_filter=RunnerFilter(checks=checks)
    )

    # then
    summary = report.get_summary()

    assert summary["passed"] == 1
    assert summary["failed"] == 2
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0

    passing_resources = {
        "tasks.block.ansible.builtin.yum.Install httpd and memcached",
    }

    failing_resources = {
        "tasks.block.ansible.builtin.yum.Install httpd and memcached",
        "block.Install, configure, and start Apache",
    }

    passed_check_resources = {check.resource for check in report.passed_checks}
    failed_check_resources = {check.resource for check in report.failed_checks}

    assert passing_resources == passed_check_resources
    assert failing_resources == failed_check_resources


@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector,
    ],
)
def test_runner_with_nested_blocks(graph_connector):
    # given
    test_file = EXAMPLES_DIR / "nested_blocks.yml"
    checks = ["CKV_ANSIBLE_1", "CKV2_ANSIBLE_3"]

    # when
    report = Runner(db_connector=graph_connector()).run(
        root_folder="", files=[str(test_file)], runner_filter=RunnerFilter(checks=checks)
    )

    # then
    summary = report.get_summary()

    # if we increase the level of nested block levels for Python checks, then this goes up to 6
    assert summary["passed"] == 4
    assert summary["failed"] == 5
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0

    passing_resources = {
        "tasks.ansible.builtin.uri.1st level uri",
        "tasks.block.ansible.builtin.uri.2nd level uri",
        "tasks.block.block.ansible.builtin.uri.3rd level uri",
        "tasks.block.block.block.ansible.builtin.uri.4th level uri",
    }

    failing_resources = {
        "block.1st level block",
        "block.block.2nd level block",
        "block.block.block.3rd level block",
        "block.block.block.block.4th level block",
        "block.block.block.block.block.5th level block",
    }

    passed_check_resources = {check.resource for check in report.passed_checks}
    failed_check_resources = {check.resource for check in report.failed_checks}

    assert passing_resources == passed_check_resources
    assert failing_resources == failed_check_resources


@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector,
    ],
)
def test_runner_with_no_tasks(graph_connector):
    # given
    test_file = EXAMPLES_DIR / "no_tasks.yml"

    # when
    report = Runner(db_connector=graph_connector()).run(root_folder="", files=[str(test_file)])

    # then
    summary = report.get_summary()

    assert summary["passed"] == 0
    assert summary["failed"] == 0
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0


@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector,
    ],
)
def test_get_resource(graph_connector):
    # given
    file_path = "/example/site.yml"
    key = '[].tasks[?"amazon.aws.ec2_instance" != null][].[].tasks[?"amazon.aws.ec2_instance" != null][].CKV_AWS_135[6:12]'
    start_line = 8
    end_line = 12
    definition = [
        {
            "name": "Verify tests",
            "hosts": "all",
            "gather_facts": False,
            "tasks": [
                {
                    "name": "enabled",
                    "amazon.aws.ec2_instance": {
                        "name": "public-compute-instance",
                        "key_name": "prod-ssh-key",
                        "vpc_subnet_id": "subnet-5ca1ab1e",
                        "instance_type": "c5.large",
                        "security_group": "default",
                        "network": {"assign_public_ip": False, "__startline__": 14, "__endline__": 15},
                        "image_id": "ami-123456",
                        "ebs_optimized": True,
                        "__startline__": 8,
                        "__endline__": 17,
                    },
                    "__startline__": 6,
                    "__endline__": 17,
                }
            ],
            "__startline__": 2,
            "__endline__": 17,
        }
    ]
    runner = Runner(db_connector=graph_connector())
    runner.definitions = {file_path: definition}

    # when
    new_key = runner.get_resource(
        file_path=file_path,
        key=key,
        supported_entities=[],
        start_line=start_line,
        end_line=end_line,
    )

    # then
    assert new_key == "tasks.amazon.aws.ec2_instance.enabled"


@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector,
    ],
)
def test_get_resource_without_name(graph_connector):
    # given
    file_path = "/example/site.yml"
    key = '[].tasks[?"amazon.aws.ec2_instance" != null][].[].tasks[?"amazon.aws.ec2_instance" != null][].CKV_AWS_135[6:12]'
    start_line = 8
    end_line = 12
    definition = [
        {
            "name": "Verify tests",
            "hosts": "all",
            "gather_facts": False,
            "tasks": [
                {
                    "amazon.aws.ec2_instance": {
                        "name": "public-compute-instance",
                        "key_name": "prod-ssh-key",
                        "vpc_subnet_id": "subnet-5ca1ab1e",
                        "instance_type": "c5.large",
                        "security_group": "default",
                        "network": {"assign_public_ip": False, "__startline__": 14, "__endline__": 15},
                        "image_id": "ami-123456",
                        "ebs_optimized": True,
                        "__startline__": 8,
                        "__endline__": 17,
                    },
                    "__startline__": 6,
                    "__endline__": 17,
                }
            ],
            "__startline__": 2,
            "__endline__": 17,
        }
    ]
    runner = Runner(db_connector=graph_connector())
    runner.definitions = {file_path: definition}

    # when
    new_key = runner.get_resource(
        file_path=file_path,
        key=key,
        supported_entities=[],
        start_line=start_line,
        end_line=end_line,
    )

    # then
    assert new_key == "tasks.amazon.aws.ec2_instance.unknown"


def test_runner_process_utf16_file():
    # given
    test_file = EXAMPLES_DIR / "k8s_utf16.yaml"

    # when
    report = Runner().run(root_folder="", files=[str(test_file)])

    # then
    summary = report.get_summary()

    assert summary["passed"] == 0
    assert summary["failed"] == 0
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0



from pathlib import Path

import yaml
from _pytest.logging import LogCaptureFixture

from checkov.common.checks_infra.checks_parser import GraphCheckParser
from checkov.common.checks_infra.resources_types import resources_types as raw_resources_types

EXAMPLES_DIR = Path(__file__).parent / "examples"


def test_validate_check_config(caplog: LogCaptureFixture):
    # given
    file_path = EXAMPLES_DIR / "valid_check.yaml"
    check_yaml = yaml.safe_load(file_path.read_text())

    # when
    valid = GraphCheckParser().validate_check_config(file_path=str(file_path), raw_check=check_yaml)

    # then
    assert valid
    assert len(caplog.messages) == 0


def test_validate_check_config_missing_metadata(caplog: LogCaptureFixture):
    # given
    file_path = EXAMPLES_DIR / "missing_metadata.yaml"
    check_yaml = yaml.safe_load(file_path.read_text())

    # when
    valid = GraphCheckParser().validate_check_config(file_path=str(file_path), raw_check=check_yaml)

    # then
    assert not valid
    assert caplog.messages == [
        f"Custom policy {file_path} is missing required fields metadata.id, metadata.name, metadata.category"
    ]


def test_validate_check_config_missing_metadata_category(caplog: LogCaptureFixture):
    # given
    file_path = EXAMPLES_DIR / "missing_metadata_category.yaml"
    check_yaml = yaml.safe_load(file_path.read_text())

    # when
    valid = GraphCheckParser().validate_check_config(file_path=str(file_path), raw_check=check_yaml)

    # then
    assert not valid
    assert caplog.messages == [f"Custom policy {file_path} is missing required fields metadata.category"]


def test_validate_check_config_missing_definition(caplog: LogCaptureFixture):
    # given
    file_path = EXAMPLES_DIR / "missing_definition.yaml"
    check_yaml = yaml.safe_load(file_path.read_text())

    # when
    valid = GraphCheckParser().validate_check_config(file_path=str(file_path), raw_check=check_yaml)

    # then
    assert not valid
    assert caplog.messages == [f"Custom policy {file_path} is missing required fields definition"]


def test_validate_check_config_invalid_definition(caplog: LogCaptureFixture):
    # given
    file_path = EXAMPLES_DIR / "invalid_definition.yaml"
    check_yaml = yaml.safe_load(file_path.read_text())

    # when
    valid = GraphCheckParser().validate_check_config(file_path=str(file_path), raw_check=check_yaml)

    # then
    assert not valid
    assert caplog.messages == [
        f"Custom policy {file_path} has an invalid 'definition' block type 'NoneType', "
        "needs to be either a 'list' or 'dict'"
    ]

def test_parse_taggable_resource_string():
    parser = GraphCheckParser()
    raw_check = {"resource_types": "taggable"}
    providers = ["aws"]
    check = parser._parse_raw_check(raw_check, [], providers)
    assert check.resource_types == raw_resources_types.get("aws_taggable")

def test_parse_taggable_resource_list():
    parser = GraphCheckParser()
    raw_check = {"resource_types": ["taggable"]}
    providers = ["azure"]
    check = parser._parse_raw_check(raw_check, [], providers)
    assert check.resource_types == raw_resources_types.get("azure_taggable")



import unittest

from pathlib import Path

from pytest_mock import MockerFixture

from checkov.common.runners.runner_registry import RunnerRegistry
from checkov.common.util.banner import banner
from checkov.runner_filter import RunnerFilter
from checkov.terraform.module_loading.content import ModuleContent
from checkov.terraform.module_loading.registry import module_loader_registry
from checkov.terraform.plan_runner import Runner as tf_plan_runner
from checkov.terraform.tf_parser import TFParser


class TestRunnerRegistryEnrichment(unittest.TestCase):
    def test_enrichment_of_plan_report(self):
        allowed_checks = ["CKV_AWS_20", "CKV_AWS_28", "CKV_AWS_63", "CKV_AWS_119"]
        runner_registry = RunnerRegistry(
            banner, RunnerFilter(checks=allowed_checks, framework=["terraform_plan"]), tf_plan_runner()
        )

        repo_root = Path(__file__).parent / "plan_with_hcl_for_enrichment"
        valid_plan_path = repo_root / "tfplan.json"

        report = runner_registry.run(repo_root_for_plan_enrichment=[repo_root], files=[str(valid_plan_path)])[0]

        failed_check_ids = {c.check_id for c in report.failed_checks}
        skipped_check_ids = {c.check_id for c in report.skipped_checks}
        expected_failed_check_ids = {"CKV_AWS_63", "CKV_AWS_119"}
        expected_skipped_check_ids = {"CKV_AWS_20", "CKV_AWS_28"}

        enriched_data = {(c.file_path, tuple(c.file_line_range), tuple(c.code_block)) for c in report.failed_checks}
        expected_enriched_data = {
            (
                "iam.tf",
                (1, 19),
                (
                    (1, 'resource "aws_iam_policy" "policy" {\n'),
                    (2, '  name        = "my_policy-123456789101"\n'),
                    (3, '  path        = "/"\n'),
                    (4, '  description = "My test policy"\n'),
                    (5, "  policy = <<EOF\n"),
                    (6, "{\n"),
                    (7, '  "Version": "2012-10-17",\n'),
                    (8, '  "Statement": [\n'),
                    (9, "    {\n"),
                    (10, '      "Action": [\n'),
                    (11, '        "*"\n'),
                    (12, "      ],\n"),
                    (13, '      "Effect": "Allow",\n'),
                    (14, '      "Resource": "arn:aws:iam::${var.aws_account_id}:role/admin"\n'),
                    (15, "    }\n"),
                    (16, "  ]\n"),
                    (17, "}\n"),
                    (18, "EOF\n"),
                    (19, "}"),
                ),
            ),
            (
                "dynamodb.tf",
                (1, 12),
                (
                    (1, 'resource "aws_dynamodb_table" "cross-environment-violations" {\n'),
                    (2, "  # checkov:skip=CKV_AWS_28: ignoring backups for now\n"),
                    (3, '  name           = "CrossEnvironmentViolations"\n'),
                    (4, "  read_capacity  = 20\n"),
                    (5, "  write_capacity = 20\n"),
                    (6, '  hash_key       = "id"\n'),
                    (7, "  attribute {\n"),
                    (8, '    name = "id"\n'),
                    (9, '    type = "S"\n'),
                    (10, "  }\n"),
                    (11, "  provider = aws.current_region\n"),
                    (12, "}"),
                ),
            ),
        }

        self.assertEqual(len(failed_check_ids), 2)
        self.assertEqual(failed_check_ids, expected_failed_check_ids)
        self.assertEqual(len(skipped_check_ids), 2)
        self.assertEqual(skipped_check_ids, expected_skipped_check_ids)
        self.assertEqual(enriched_data, expected_enriched_data)

    def test_enrichment_of_plan_report_with_modules(self):
        allowed_checks = ["CKV_AWS_66", "CKV_AWS_158"]
        runner_registry = RunnerRegistry(
            banner, RunnerFilter(checks=allowed_checks, framework=["terraform_plan"]), tf_plan_runner()
        )

        repo_root = Path(__file__).parent / "plan_with_tf_modules_for_enrichment"
        valid_plan_path = repo_root / "tfplan.json"

        report = runner_registry.run(repo_root_for_plan_enrichment=[repo_root], files=[str(valid_plan_path)])[0]

        failed_check_ids = [c.check_id for c in report.failed_checks]
        passed_check_ids = [c.check_id for c in report.passed_checks]
        skipped_check_ids = [c.check_id for c in report.skipped_checks]
        expected_failed_check_ids = ["CKV_AWS_158", "CKV_AWS_158"]
        expected_passed_check_ids = ["CKV_AWS_66", "CKV_AWS_66"]
        expected_skipped_check_ids = []

        enriched_data = set(
            [(c.file_path, tuple(c.file_line_range), tuple(c.code_block)) for c in report.failed_checks]
        )
        expected_enriched_data = {
            (f"/{Path.relative_to(valid_plan_path, repo_root)}", (16, 16), (),),
            (
                "log_group/main.tf",
                (1, 2),
                ((1, 'resource "aws_cloudwatch_log_group" "not_encrypted" {\n'), (2, "}\n"),),
            ),
        }

        self.assertEqual(len(failed_check_ids), 2)
        self.assertEqual(failed_check_ids, expected_failed_check_ids)
        self.assertEqual(len(passed_check_ids), 2)
        self.assertEqual(passed_check_ids, expected_passed_check_ids)
        self.assertEqual(len(skipped_check_ids), 0)
        self.assertEqual(skipped_check_ids, expected_skipped_check_ids)
        self.assertEqual(enriched_data, expected_enriched_data)

    def test_enrichment_of_plan_report_with_for_each(self):
        allowed_checks = ["CKV2_AWS_38", "CKV_AWS_277"]
        runner_registry = RunnerRegistry(
            banner, RunnerFilter(checks=allowed_checks, framework=["terraform_plan"]), tf_plan_runner()
        )

        repo_root = Path(__file__).parent / "plan_with_for_each_for_enrichment"
        valid_plan_path = repo_root / "tf_plan.json"

        report = runner_registry.run(repo_root_for_plan_enrichment=[repo_root], files=[str(valid_plan_path)])[0]

        self.assertEqual(len(report.failed_checks), 0)

        self.assertEqual(len(report.passed_checks), 0)

        self.assertEqual(len(report.skipped_checks), 5)


    def test_skip_check(self):
        allowed_checks = ["CKV_AWS_20", "CKV_AWS_28"]
        runner_registry = RunnerRegistry(
            banner, RunnerFilter(checks=allowed_checks, framework=["terraform_plan"]), tf_plan_runner()
        )

        repo_root = Path(__file__).parent / "plan_with_hcl_for_enrichment"
        valid_plan_path = repo_root / "tfplan.json"

        report = runner_registry.run(repo_root_for_plan_enrichment=[repo_root], files=[str(valid_plan_path)])[0]

        failed_check_ids = {c.check_id for c in report.failed_checks}
        skipped_check_ids = {c.check_id for c in report.skipped_checks}
        expected_skipped_check_ids = {"CKV_AWS_20", "CKV_AWS_28"}

        self.assertEqual(len(failed_check_ids), 0)
        self.assertEqual(len(skipped_check_ids), 2)
        self.assertEqual(skipped_check_ids, expected_skipped_check_ids)


    def test_skip_check_in_module(self):
        allowed_checks = ["CKV_AWS_19", "CKV2_AWS_6"]
        runner_registry = RunnerRegistry(
            banner, RunnerFilter(checks=allowed_checks, framework=["terraform_plan"]), tf_plan_runner()
        )

        repo_root = Path(__file__).parent / "plan_module_skip_for_enrichment" / "tf"
        valid_plan_path = repo_root / "tfplan.json"

        report = runner_registry.run(repo_root_for_plan_enrichment=[repo_root], files=[str(valid_plan_path)])[0]

        failed_check_ids = {c.check_id for c in report.failed_checks}
        skipped_check_ids = {c.check_id for c in report.skipped_checks}
        expected_skipped_check_ids = { "CKV2_AWS_6" }

        self.assertEqual(len(failed_check_ids), 0)
        self.assertEqual(len(skipped_check_ids), 1)
        self.assertEqual(skipped_check_ids, expected_skipped_check_ids)


def test_enrichment_of_plan_report_with_external_modules(mocker: MockerFixture):
    # given
    allowed_checks = ["CKV_AWS_66", "CKV_AWS_158"]
    runner_filter = RunnerFilter(
        checks=allowed_checks,
        framework=["terraform_plan"],
        download_external_modules=True,
        external_modules_download_path="example/path",
    )
    runner_registry = RunnerRegistry(banner, runner_filter, tf_plan_runner())

    repo_root = Path(__file__).parent / "plan_with_external_tf_modules_for_enrichment"
    valid_plan_path = repo_root / "tfplan.json"

    def _load_tf_modules(*args, **kwargs):
        # set module cache to be the local folder instead of downloading the external module
        module_loader_registry.module_content_cache = {
            'terraform-aws-modules/cloudwatch/aws//modules/log-group:2.1.0': ModuleContent(
                dir=str(repo_root / "log_group_external")
            )
        }

    parse_directory_spy = mocker.spy(TFParser, "parse_directory")
    mocker.patch("checkov.terraform.tf_parser.load_tf_modules", side_effect=_load_tf_modules)

    # when
    report = runner_registry.run(repo_root_for_plan_enrichment=[repo_root], files=[str(valid_plan_path)])[0]

    # reset module cache
    module_loader_registry.reset_module_content_cache()

    # then
    summary = report.get_summary()

    assert summary["passed"] == 1
    assert summary["failed"] == 0
    assert summary["skipped"] == 1
    assert summary["parsing_errors"] == 0

    assert {c.check_id for c in report.passed_checks} == {"CKV_AWS_66"}
    assert {c.check_id for c in report.skipped_checks} == {"CKV_AWS_158"}

    parse_directory_spy.assert_called()
    call_args = parse_directory_spy.call_args
    assert call_args.kwargs["external_modules_download_path"] == "example/path"


if __name__ == "__main__":
    unittest.main()



from constructs import Construct
from aws_cdk import App, Stack 
from aws_cdk import (
    aws_s3 as s4
)

class MyStack(Stack):

    def __init__(self, scope: Construct, id: str, **kwargs):
        super().__init__(scope, id, **kwargs)

		# fail
        bucket = s4.Bucket(self, "MyS3Bucket",
            bucket_name='my-s3-bucket',
            public_read_access=False,
            block_public_access=s4.BlockPublicAccess(
                ignore_public_acls=False
            )
        )

        value = False
		# fail
        bucket2 = s4.Bucket(self, "MyS3Bucket2",
            bucket_name='my-s3-bucket2',
            public_read_access=False,
            block_public_access=s4.BlockPublicAccess(
                ignore_public_acls=value
            )
        )

app = App()
MyStack(app, "my-stack-name")

app.synth()



expected_postgres_module = {"create": True,
                            "name": "${var.name}",
                            "use_name_prefix": True,
                            "description": "Security Group managed by Terraform",
                            "vpc_id": "${var.vpc_id}",
                            "revoke_rules_on_delete": False,
                            "tags": {},
                            "ingress_rules": ["postgresql-tcp"],
                            "ingress_with_self": [{"rule": "all-all"}],
                            "number_of_computed_egress_rules": 0,
                            }

expected_terragoat_local_resource_prefix = {'resource_prefix': {'value': 'test_id-acme-dev'}}

expected_terragoat_db_instance = {'name': 'db1',
                                  'engine': 'mysql',
                                  'option_group_name': 'og-test_id-acme-dev',
                                  'parameter_group_name': 'pg-test_id-acme-dev',
                                  'db_subnet_group_name': 'sg-test_id-acme-dev',
                                  'vpc_security_group_ids': ['aws_security_group.default.id'],
                                  'identifier': 'rds-test_id-acme-dev',
                                  'password': 'Aa1234321Bb',
                                  'tags': {'Name': 'test_id-acme-dev-rds',
                                            'Environment': 'test_id-acme-dev'}
                                  }


expected_eks = {
    "resource": {
        "aws_eks_cluster.tf_eks": {
            "version": ["1.19"],
            "vpc_config": {
                "security_group_ids": ["aws_security_group.master.id"],
                "subnet_ids": "aws_subnet.eks[*].id"
            },
        }
    }
}


expected_provider = {
    "provider": {
        "aws": {
            "access_key": ["AKIAVAN"],
            "secret_key": ["0CU4jk0"],
            "region": ["us-west-2"],
        }
    }
}



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.APIServicesUseVirtualNetwork import check
from checkov.common.models.enums import CheckResult


class TestAPIServicesUseVirtualNetwork(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
                resource "azurerm_api_management" "example" {
                  name                = "example-apim"
                  location            = azurerm_resource_group.example.location
                  resource_group_name = azurerm_resource_group.example.name
                  publisher_name      = "My Company"
                  publisher_email     = "company@terraform.io"
                
                  sku_name = "Developer_1"
                
                  policy {
                    xml_content = <<XML
                    <policies>
                      <inbound />
                      <backend />
                      <outbound />
                      <on-error />
                    </policies>
                XML
                
                  }
                }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_api_management']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
                resource "azurerm_api_management" "example" {
                  name                = "example-apim"
                  location            = azurerm_resource_group.example.location
                  resource_group_name = azurerm_resource_group.example.name
                  publisher_name      = "My Company"
                  publisher_email     = "company@terraform.io"
                
                  sku_name = "Developer_1"
                  virtual_network_configuration {
                    subnet_id = azure_subnet.subnet_not_public_ip.id
                  }
                  policy {
                    xml_content = <<XML
                    <policies>
                      <inbound />
                      <backend />
                      <outbound />
                      <on-error />
                    </policies>
                XML
                
                  }
                }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_api_management']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from aws_cdk import core
from aws_cdk import aws_apigateway as apigw

class MyApiGatewayMethodStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create the API Gateway Method based on the conditions
        api_method = apigw.Method(
            self, 'MyApiGatewayMethod',
            http_method='GET',  # Replace with your desired HTTP method
            resource=self.node.try_get_context('resource'),  # Replace with your API resource
            rest_api=self.node.try_get_context('rest_api'),  # Replace with your REST API
            authorization_type=apigw.AuthorizationType.NONE,  # Set the AuthorizationType to NONE
            api_key_required=False  # Set ApiKeyRequired to false
            # You can add other properties as needed for your method
        )

app = core.App()
MyApiGatewayMethodStack(app, "MyApiGatewayMethodStack")
app.synth()



import os
import shutil
import unittest
from pathlib import Path
from unittest import mock

import pytest

from checkov.common.util.consts import DEFAULT_EXTERNAL_MODULES_DIR
from checkov.terraform.modules.module_objects import TFDefinitionKey, TFModule
from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph
from checkov.terraform.tf_parser import TFParser


@pytest.fixture
def tmp_path(request, tmp_path: Path):
    # https://pytest.org/en/latest/how-to/unittest.html#mixing-pytest-fixtures-into-unittest-testcase-subclasses-using-marks
    request.cls.tmp_path = tmp_path


@pytest.mark.usefixtures("tmp_path")
class TestParserInternals(unittest.TestCase):
    expected_source_modules = {0: set(), 1: set(), 2: set(), 3: {0}, 4: {0}, 5: {0}, 6: {0}, 7: {1}, 8: {1}, 9: {1},
                               10: {1}, 11: {3}, 12: {3}, 13: {3}, 14: {7}, 15: {7}, 16: {7}, 17: {4}, 18: {4}, 19: {4},
                               20: {8}, 21: {8}, 22: {8}, 23: {8}, 24: {8}}

    def setUp(self) -> None:
        from checkov.terraform.module_loading.registry import ModuleLoaderRegistry

        # needs to be reset, because the cache belongs to the class not instance
        ModuleLoaderRegistry.module_content_cache = {}

        self.resources_dir = os.path.realpath(os.path.join(os.path.dirname(os.path.realpath(__file__)), "./resources"))
        self.external_module_path = ''

    def tearDown(self) -> None:
        if os.path.exists(self.external_module_path):
            shutil.rmtree(self.external_module_path)

    def test_load_inner_registry_module_new_parser(self):
        parser = TFParser()
        directory = os.path.join(self.resources_dir, "registry_security_group_inner_module")
        self.external_module_path = os.path.join(self.tmp_path, DEFAULT_EXTERNAL_MODULES_DIR)
        out_definitions = parser.parse_directory(
            directory=directory,
            out_evaluations_context={},
            download_external_modules=True,
            external_modules_download_path=self.external_module_path)
        self.assertEqual(11, len(list(out_definitions.keys())))
        expected_remote_module_path = f'{self.external_module_path}/github.com/terraform-aws-modules/terraform-aws-security-group/ff2efb814c924572d27280b99a799fc34d061109'
        expected_inner_remote_module_path = f'{expected_remote_module_path}/modules/http-80'
        expected_main_file = os.path.join(directory, 'main.tf')

        assert TFDefinitionKey(file_path=expected_main_file) in out_definitions

        assert TFDefinitionKey(file_path=f"{expected_inner_remote_module_path}/auto_values.tf", tf_source_modules=TFModule(name='web_server_sg', path=expected_main_file)) in out_definitions
        assert TFDefinitionKey(file_path=f"{expected_inner_remote_module_path}/main.tf", tf_source_modules=TFModule(name='web_server_sg', path=expected_main_file)) in out_definitions
        assert TFDefinitionKey(file_path=f"{expected_inner_remote_module_path}/outputs.tf", tf_source_modules=TFModule(name='web_server_sg', path=expected_main_file)) in out_definitions
        assert TFDefinitionKey(file_path=f"{expected_inner_remote_module_path}/variables.tf", tf_source_modules=TFModule(name='web_server_sg', path=expected_main_file)) in out_definitions
        assert TFDefinitionKey(file_path=f"{expected_inner_remote_module_path}/versions.tf", tf_source_modules=TFModule(name='web_server_sg', path=expected_main_file)) in out_definitions

        assert TFDefinitionKey(file_path=f"{expected_remote_module_path}/main.tf", tf_source_modules=TFModule(name='sg', path=f"{expected_inner_remote_module_path}/main.tf", nested_tf_module=TFModule(path=expected_main_file, name='web_server_sg'))) in out_definitions
        assert TFDefinitionKey(file_path=f"{expected_remote_module_path}/outputs.tf", tf_source_modules=TFModule(name='sg', path=f"{expected_inner_remote_module_path}/main.tf", nested_tf_module=TFModule(path=expected_main_file, name='web_server_sg'))) in out_definitions
        assert TFDefinitionKey(file_path=f"{expected_remote_module_path}/rules.tf", tf_source_modules=TFModule(name='sg', path=f"{expected_inner_remote_module_path}/main.tf", nested_tf_module=TFModule(path=expected_main_file, name='web_server_sg'))) in out_definitions
        assert TFDefinitionKey(file_path=f"{expected_remote_module_path}/variables.tf", tf_source_modules=TFModule(name='sg', path=f"{expected_inner_remote_module_path}/main.tf", nested_tf_module=TFModule(path=expected_main_file, name='web_server_sg'))) in out_definitions
        assert TFDefinitionKey(file_path=f"{expected_remote_module_path}/versions.tf", tf_source_modules=TFModule(name='sg', path=f"{expected_inner_remote_module_path}/main.tf", nested_tf_module=TFModule(path=expected_main_file, name='web_server_sg'))) in out_definitions

    def test_invalid_module_sources_new_parser(self):
        parser = TFParser()
        directory = os.path.join(self.resources_dir, "failing_module_address")
        self.external_module_path = os.path.join(directory, DEFAULT_EXTERNAL_MODULES_DIR)
        out_definitions = parser.parse_directory(
            directory=directory,
            out_evaluations_context={},
            download_external_modules=True,
            external_modules_download_path=DEFAULT_EXTERNAL_MODULES_DIR)
        # check that only the original file was parsed successfully without getting bad external modules
        self.assertEqual(1, len(list(out_definitions.keys())))

    def test_malformed_output_blocks_new_parser(self):
        parser = TFParser()
        directory = os.path.join(self.resources_dir, "malformed_outputs")
        self.external_module_path = os.path.join(directory, DEFAULT_EXTERNAL_MODULES_DIR)
        out_definitions = parser.parse_directory(
            directory=directory,
            out_evaluations_context={},
            download_external_modules=True,
            external_modules_download_path=DEFAULT_EXTERNAL_MODULES_DIR)
        file_path, entity_definitions = next(iter(out_definitions.items()))
        self.assertEqual(2, len(list(out_definitions[file_path]['output'])))

    def test_load_local_module_new_parser(self):
        parser = TFParser()
        directory = os.path.join(self.resources_dir, "local_module")
        out_definitions = parser.parse_directory(directory=directory, out_evaluations_context={})

        self.assertEqual(len(out_definitions), 3)
        self.assertEqual(len(parser.loaded_files_map), 2)

        local_module_path = os.path.join(directory, 'main.tf')
        module_path = os.path.join(directory, "module/main.tf")
        main_key = TFDefinitionKey(file_path=local_module_path)
        key_idx_0 = TFDefinitionKey(file_path=module_path, tf_source_modules=TFModule(path=local_module_path, name='mod'))
        key_idx_1 = TFDefinitionKey(file_path=module_path, tf_source_modules=TFModule(path=local_module_path, name='mod2'))

        assert main_key in out_definitions
        assert key_idx_0 in out_definitions
        assert key_idx_1 in out_definitions
        assert out_definitions[main_key]['module'][0]['mod']['__resolved__'] == [key_idx_0]
        assert out_definitions[main_key]['module'][1]['mod2']['__resolved__'] == [key_idx_1]

        assert parser.external_modules_source_map == {(os.path.join(directory, 'module'), 'latest'): os.path.join(directory, 'module')}
        assert parser.external_vars == {}
        assert parser.keys_to_remove == {TFDefinitionKey(file_path=module_path)}
        assert parser._parsed_directories == {
            directory,
            os.path.join(directory, 'module')
        }

    def test_load_nested_module_new_parser(self):
        parser = TFParser()
        directory = os.path.join(self.resources_dir, "parser_nested_modules")
        o_definitions = parser.parse_directory(directory=directory, out_evaluations_context={})

        self.assertEqual(len(o_definitions), 5)
        self.assertEqual(len(parser.loaded_files_map), 5)

        main_module_path = os.path.join(directory, 'main.tf')
        module2_main_path = os.path.join(directory, 'module/module2/main.tf')
        module2_var_path = os.path.join(directory, 'module/module2/variable.tf')
        module1_main_path = os.path.join(directory, 'module/main.tf')
        module1_var_path = os.path.join(directory, 'module/variable.tf')

        main_module = TFDefinitionKey(file_path=main_module_path)
        module_main_key = TFDefinitionKey(file_path=module1_main_path, tf_source_modules=TFModule(path=main_module_path, name='s3_module'))
        module_var_key = TFDefinitionKey(file_path=module1_var_path, tf_source_modules=TFModule(path=main_module_path, name='s3_module'))
        module2_main_key = TFDefinitionKey(file_path=module2_main_path, tf_source_modules=TFModule(path=module1_main_path, name='inner_s3_module', nested_tf_module=TFModule(path=main_module_path, name='s3_module')))
        module2_var_key = TFDefinitionKey(file_path=module2_var_path, tf_source_modules=TFModule(path=module1_main_path, name='inner_s3_module', nested_tf_module=TFModule(path=main_module_path, name='s3_module')))

        assert main_module in o_definitions
        assert module_main_key in o_definitions
        assert module_var_key in o_definitions
        assert module2_main_key in o_definitions
        assert module2_var_key in o_definitions

        assert o_definitions[main_module]['module'][0]['s3_module']['__resolved__'] == [module_main_key, module_var_key]
        assert o_definitions[module_main_key]['module'][0]['inner_s3_module']['__resolved__'] == [module2_main_key, module2_var_key]

    def test_load_nested_dup_module(self):
        parser = TFParser()
        directory = os.path.join(self.resources_dir, "parser_dup_nested")
        o_definitions = parser.parse_directory(directory=directory, out_evaluations_context={})

        self.assertEqual(len(o_definitions), 7)
        self.assertEqual(len(parser.loaded_files_map), 3)

        main_module_path = os.path.join(directory, 'main.tf')
        module1_path = os.path.join(directory, 'module/main.tf')
        module2_path = os.path.join(directory, 'module/module2/main.tf')

        main_module = TFDefinitionKey(file_path=main_module_path)
        module1_key0 = TFDefinitionKey(file_path=module1_path, tf_source_modules=TFModule(path=main_module_path, name='s3_module'))
        module1_key1 = TFDefinitionKey(file_path=module1_path, tf_source_modules=TFModule(path=main_module_path, name='s3_module2'))
        module2_key0_nest0 = TFDefinitionKey(file_path=module2_path, tf_source_modules=TFModule(path=module1_path, name='inner_s3_module', nested_tf_module=TFModule(path=main_module_path, name='s3_module')))
        module2_key1_nest0 = TFDefinitionKey(file_path=module2_path, tf_source_modules=TFModule(path=module1_path, name='inner_s3_module2', nested_tf_module=TFModule(path=main_module_path, name='s3_module')))
        module2_key0_nest1 = TFDefinitionKey(file_path=module2_path, tf_source_modules=TFModule(path=module1_path, name='inner_s3_module', nested_tf_module=TFModule(path=main_module_path, name='s3_module2')))
        module2_key1_nest1 = TFDefinitionKey(file_path=module2_path, tf_source_modules=TFModule(path=module1_path, name='inner_s3_module2', nested_tf_module=TFModule(path=main_module_path, name='s3_module2')))

        assert main_module in o_definitions
        assert module1_key0 in o_definitions
        assert module1_key1 in o_definitions
        assert module2_key0_nest1 in o_definitions
        assert module2_key1_nest1 in o_definitions
        assert module2_key0_nest0 in o_definitions
        assert module2_key1_nest0 in o_definitions

    def test_tf_parser(self):
        parser = TFParser()
        directory = os.path.join(self.resources_dir, "parser_dup_nested")
        module, tf_definitions = parser.parse_hcl_module(source_dir=directory, source='terraform')

        local_graph = TerraformLocalGraph(module)
        local_graph.build_graph(render_variables=True)

        for i, vertex in enumerate(local_graph.vertices):
            assert vertex.source_module == self.expected_source_modules[i]

        assert len(local_graph.edges) == 20

        assert module
        assert tf_definitions

    def test_parser_with_tvars(self):
        parser = TFParser()
        directory = os.path.join(self.resources_dir, "parser_tfvars")
        module, tf_definitions = parser.parse_hcl_module(source_dir=directory, source='terraform')
        assert module

    def test_backtrack_module(self):
        # given
        directory = os.path.join(self.resources_dir, "parse_backtrack_module/example")

        # when
        module, tf_definitions = TFParser().parse_hcl_module(
            source_dir=directory,
            source="terraform",
            download_external_modules=False,  # important to keep it 'False'
        )

        # then
        assert module
        assert len(tf_definitions) == 2  # need to be 2 files (the module reference and the actual module content)



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.StorageAccountLoggingQueueServiceEnabled import check
from checkov.common.models.enums import CheckResult


class TestStorageAccountLoggingQueueServiceEnabled(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_storage_account" "example" {
              name                     = "example"
              resource_group_name      = data.azurerm_resource_group.example.name
              location                 = data.azurerm_resource_group.example.location
              account_tier             = "Standard"
              account_replication_type = "GRS"
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_storage_account']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_2(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_storage_account" "example" {
              name                     = "example"
              resource_group_name      = data.azurerm_resource_group.example.name
              location                 = data.azurerm_resource_group.example.location
              account_tier             = "Standard"
              account_replication_type = "GRS"
              queue_properties  {
                logging {
                }
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_storage_account']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_storage_account" "example" {
              name                     = "example"
              resource_group_name      = data.azurerm_resource_group.example.name
              location                 = data.azurerm_resource_group.example.location
              account_tier             = "Standard"
              account_replication_type = "GRS"
              queue_properties  {
                logging {
                  delete                = true
                  read                  = true
                  write                 = true
                  version               = "1.0"
                  retention_policy_days = 10
                }
                hour_metrics {
                  enabled               = true
                  include_apis          = true
                  version               = "1.0"
                  retention_policy_days = 10
                }
                minute_metrics {
                  enabled               = true
                  include_apis          = true
                  version               = "1.0"
                  retention_policy_days = 10
                }
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_storage_account']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_blobstorage(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_storage_account" "example" {
              name                     = "example"
              resource_group_name      = data.azurerm_resource_group.example.name
              location                 = data.azurerm_resource_group.example.location
              account_tier             = "Standard"
              account_replication_type = "LRS"
              account_kind             = "BlobStorage"
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_storage_account']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest
from pathlib import Path

from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph
from checkov.terraform.tf_parser import TFParser


class TestModuleProvider(unittest.TestCase):
    def test_module_with_two_providers(self):
        test_files_dir = Path(__file__).parent / "example_ModuleProvider"

        hcl_config_parser = TFParser()
        module, _ = hcl_config_parser.parse_hcl_module(test_files_dir, source='TERRAFORM')
        local_graph = TerraformLocalGraph(module)
        local_graph.build_graph(True)

        assert len(local_graph.edges) == 2
        assert local_graph.vertices[0].attributes.get('__provider_address__') == local_graph.vertices[3].attributes.get('__address__')
        assert local_graph.vertices[0].attributes.get('__provider_address__') == local_graph.vertices[3].config['aws'].get('__address__')
        assert local_graph.edges[0].origin == 1 and local_graph.edges[0].dest == 3
        assert local_graph.edges[1].origin == 0 and local_graph.edges[1].dest == 3

    def test_module_with_one_def_provider(self):
        test_files_dir = Path(__file__).parent / "example_module_with_one_provider"

        hcl_config_parser = TFParser()
        module, _ = hcl_config_parser.parse_hcl_module(test_files_dir, source='TERRAFORM')
        local_graph = TerraformLocalGraph(module)
        local_graph.build_graph(True)

        resource_provider_address = local_graph.vertices[0].attributes.get('__provider_address__')
        default_provider_address_from_module = local_graph.vertices[2].attributes.get('__address__')

        assert resource_provider_address == default_provider_address_from_module

    def test_resource_with_def_provider(self):
        test_files_dir = Path(__file__).parent / "example_provider_without_module"

        hcl_config_parser = TFParser()
        module, _ = hcl_config_parser.parse_hcl_module(test_files_dir, source='TERRAFORM')
        local_graph = TerraformLocalGraph(module)
        local_graph.build_graph(True)

        # assert resource with provider ref.
        resource_provider_address_with_alias = local_graph.vertices[2].attributes.get('__provider_address__')
        provider_address_with_alias = local_graph.vertices[1].attributes.get('__address__')
        assert resource_provider_address_with_alias == provider_address_with_alias

        # assert resource without ref to the default provider
        resource_provider_address_default = local_graph.vertices[3].attributes.get('__provider_address__')
        provider_address_default = local_graph.vertices[0].attributes.get('__address__')
        assert resource_provider_address_default == provider_address_default

    def test_provider_nested_module(self):
        test_files_dir = Path(__file__).parent / "example_provider_with_nested_module"

        hcl_config_parser = TFParser()
        module, _ = hcl_config_parser.parse_hcl_module(test_files_dir, source='TERRAFORM')
        local_graph = TerraformLocalGraph(module)
        local_graph.build_graph(True)

        # assert resource with provider ref.
        resource_provider_address_with_alias = local_graph.vertices[2].attributes.get('__provider_address__')
        provider_address_with_alias = local_graph.vertices[1].attributes.get('__address__')
        assert resource_provider_address_with_alias == provider_address_with_alias

    def test_example_provider_with_nested_module_assign_provider(self):
        test_files_dir = Path(__file__).parent / "example_provider_with_nested_module_assign_provider"

        hcl_config_parser = TFParser()
        module, _ = hcl_config_parser.parse_hcl_module(test_files_dir, source='TERRAFORM')
        local_graph = TerraformLocalGraph(module)
        local_graph.build_graph(True)

        # assert resource with provider ref.
        resource_provider_address_with_alias = local_graph.vertices[0].attributes.get('__provider_address__')
        provider_address_with_alias = local_graph.vertices[4].attributes.get('__address__')
        assert resource_provider_address_with_alias == provider_address_with_alias

    def test_provider_edge_cases(self):
        test_files_dir = Path(__file__).parent / "example_provider_edge_case"

        hcl_config_parser = TFParser()
        module, _ = hcl_config_parser.parse_hcl_module(test_files_dir, source='TERRAFORM')
        local_graph = TerraformLocalGraph(module)
        local_graph.build_graph(True)

        assert local_graph.vertices[3].attributes.get('__provider_address__') == "aws.default"
        assert local_graph.vertices[8].attributes.get('__provider_address__') == "module.level1.aws.default"
        assert local_graph.vertices[9].attributes.get('__provider_address__') == "module.level1.aws.default"
        assert local_graph.vertices[10].attributes.get('__provider_address__') == "module.level1.aws.eu_west"
        assert local_graph.vertices[11].attributes.get('__provider_address__') == "aws.default"


if __name__ == "__main__":
    unittest.main()



import os
import unittest
from unittest.mock import patch

from checkov.common.models.enums import ParallelizationType
from checkov.common.parallelizer.parallel_runner import ParallelRunner

class TestParallel(unittest.TestCase):
    # Windows os tests
    @patch.dict(os.environ, {'PYCHARM_HOSTED': '0'})
    def test_default_for_windows(self) -> None:
        with unittest.mock.patch('platform.system', return_value='Windows'):
            parallel_runner = ParallelRunner()
            self.assertEqual(parallel_runner.type, ParallelizationType.THREAD)

    @patch.dict(os.environ, {'PYCHARM_HOSTED': '0', 'CHECKOV_PARALLELIZATION_TYPE': 'none'})
    def test_windows_with_override_to_none(self) -> None:
        with unittest.mock.patch('platform.system', return_value='Windows'):
            parallel_runner = ParallelRunner()
            self.assertEqual(parallel_runner.type, ParallelizationType.NONE)

    @patch.dict(os.environ, {'PYCHARM_HOSTED': '0', 'CHECKOV_PARALLELIZATION_TYPE': 'fork'})
    def test_windows_with_override_to_fork(self) -> None:
        # fork and spawn is not supporting by Windows
        with unittest.mock.patch('platform.system', return_value='Windows'):
            parallel_runner = ParallelRunner()
            self.assertEqual(parallel_runner.type, ParallelizationType.FORK)

    @patch.dict(os.environ, {'PYCHARM_HOSTED': '0'})
    def test_windows_with_explicitly_to_spawn(self) -> None:
        # fork and spawn is not supporting by Windows
        with unittest.mock.patch('platform.system', return_value='Windows'):
            parallel_runner = ParallelRunner(parallelization_type=ParallelizationType.SPAWN)
            self.assertEqual(parallel_runner.type, ParallelizationType.THREAD)

    @patch.dict(os.environ, {'PYCHARM_HOSTED': '0'})
    def test_windows_with_explicitly_to_none(self) -> None:
        with unittest.mock.patch('platform.system', return_value='Darwin'):
            parallel_runner = ParallelRunner(parallelization_type=ParallelizationType.NONE)
            self.assertEqual(parallel_runner.type, ParallelizationType.NONE)

    # macOS os tests
    @patch.dict(os.environ, {'PYCHARM_HOSTED': '0'})
    def test_mac_default(self) -> None:
        with unittest.mock.patch('platform.system', return_value='Darwin'):
            parallel_runner = ParallelRunner()
            self.assertEqual(parallel_runner.type, ParallelizationType.THREAD)

    @patch.dict(os.environ, {'PYCHARM_HOSTED': '0', 'CHECKOV_PARALLELIZATION_TYPE': 'none'})
    def test_mac_with_override_to_none(self) -> None:
        with unittest.mock.patch('platform.system', return_value='Darwin'):
            parallel_runner = ParallelRunner()
            self.assertEqual(parallel_runner.type, ParallelizationType.NONE)

    @patch.dict(os.environ, {'PYCHARM_HOSTED': '0', 'CHECKOV_PARALLELIZATION_TYPE': 'fork'})
    def test_mac_with_override_to_fork(self) -> None:
        # fork and spawn is not supporting by macOS
        with unittest.mock.patch('platform.system', return_value='Darwin'):
            parallel_runner = ParallelRunner()
            self.assertEqual(parallel_runner.type, ParallelizationType.FORK)

    @patch.dict(os.environ, {'PYCHARM_HOSTED': '0'})
    def test_mac_with_explicitly_to_spawn(self) -> None:
        # fork and spawn is not supporting by macOS
        with unittest.mock.patch('platform.system', return_value='Darwin'):
            parallel_runner = ParallelRunner(parallelization_type=ParallelizationType.SPAWN)
            self.assertEqual(parallel_runner.type, ParallelizationType.THREAD)

    @patch.dict(os.environ, {'PYCHARM_HOSTED': '0'})
    def test_mac_with_explicitly_to_none(self) -> None:
        with unittest.mock.patch('platform.system', return_value='Darwin'):
            parallel_runner = ParallelRunner(parallelization_type=ParallelizationType.NONE)
            self.assertEqual(parallel_runner.type, ParallelizationType.NONE)

    # general tests
    @patch.dict(os.environ, {'PYCHARM_HOSTED': '0'})
    def test_default_linux(self) -> None:
        with unittest.mock.patch('platform.system', return_value='Linux'):
            parallel_runner = ParallelRunner()
            self.assertEqual(parallel_runner.type, ParallelizationType.FORK)

    @patch.dict(os.environ, {'PYCHARM_HOSTED': '0', 'CHECKOV_PARALLELIZATION_TYPE': 'spawn'})
    def test_linux_override_by_env_param(self) -> None:
        with unittest.mock.patch('platform.system', return_value='Linux'):
            parallel_runner = ParallelRunner()
            self.assertEqual(parallel_runner.type, ParallelizationType.SPAWN)

    @patch.dict(os.environ, {'PYCHARM_HOSTED': '0'})
    def test_linux_override_by_incoming_param(self) -> None:
        with unittest.mock.patch('platform.system', return_value='Linux'):
            parallel_runner = ParallelRunner(parallelization_type=ParallelizationType.SPAWN)
            self.assertEqual(parallel_runner.type, ParallelizationType.SPAWN)

    @patch.dict(os.environ, {'PYCHARM_HOSTED': '1'})
    def test_linux_running_by_pycharm(self) -> None:
        with unittest.mock.patch('platform.system', return_value='Linux'):
            parallel_runner = ParallelRunner()
            self.assertEqual(parallel_runner.type, ParallelizationType.NONE)

    @patch.dict(os.environ, {'PYCHARM_HOSTED': '1', 'CHECKOV_PARALLELIZATION_TYPE': 'spawn'})
    def test_linux_running_by_pycharm_override_by_env_param(self) -> None:
        with unittest.mock.patch('platform.system', return_value='Linux'):
            parallel_runner = ParallelRunner()
            self.assertEqual(parallel_runner.type, ParallelizationType.SPAWN)

if __name__ == "__main__":
    unittest.main()



import argparse
import json
import shutil
import unittest

import logging
import sys
import os
import io
from pathlib import Path
from unittest.mock import patch

from _pytest.capture import CaptureFixture
from checkov.cdk.runner import CdkRunner

from checkov.common.models.enums import CheckResult
from checkov.common.output.extra_resource import ExtraResource

from checkov.cloudformation.runner import Runner as cfn_runner
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.code_categories import CodeCategoryMapping
from checkov.common.output.report import Report
from checkov.common.runners.runner_registry import RunnerRegistry
from checkov.common.util.banner import banner
from checkov.kubernetes.runner import Runner as k8_runner
from checkov.main import DEFAULT_RUNNERS
from checkov.runner_filter import RunnerFilter
from checkov.sca_package_2.runner import Runner as sca_package_runner_2
from checkov.terraform.runner import Runner as tf_runner
from checkov.bicep.runner import Runner as bicep_runner
from checkov.sast.runner import Runner as SastRunner
from checkov.terraform.runner import Runner as TFRunner
import re


logger = logging.getLogger()
logger.level = logging.INFO

class TestRunnerRegistry(unittest.TestCase):
    def test_multi_iac(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = current_dir + "/example_multi_iac"
        runner_filter = RunnerFilter(framework=None, checks=None, skip_checks=None)
        runner_registry = RunnerRegistry(
            banner, runner_filter, tf_runner(), cfn_runner(), k8_runner()
        )
        reports = runner_registry.run(root_folder=test_files_dir)
        for report in reports:
            self.assertGreater(len(report.passed_checks), 1)

    def test_resource_counts(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = current_dir + "/example_multi_iac"
        runner_filter = RunnerFilter(framework=None, checks=None, skip_checks=None)
        runner_registry = RunnerRegistry(
            banner, runner_filter, tf_runner(), cfn_runner(), k8_runner()
        )
        reports = runner_registry.run(root_folder=test_files_dir)

        # The number of resources that will get scan results. Note that this may change if we add policies covering new resource types.
        counts_by_type = {"kubernetes": 14, "terraform": 3, "cloudformation": 4}

        for report in reports:
            self.assertEqual(
                counts_by_type[report.check_type],
                report.get_summary()["resource_count"],
            )

    def test_empty_tf(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = current_dir + "/example_empty_tf"
        self.verify_empty_report(test_files_dir=test_files_dir)
        test_files = [test_files_dir + "/example_empty_file.tf"]
        self.verify_empty_report(test_files_dir=None, files=test_files)

    def test_empty_non_existing(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = current_dir + "/foo"
        self.verify_empty_report(test_files_dir=test_files_dir)
        test_files = [test_files_dir + "/goo.yaml"]
        self.verify_empty_report(test_files_dir=None, files=test_files)

    def test_empty_yaml(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = current_dir + "/example_empty_yaml"
        self.verify_empty_report(test_files_dir=test_files_dir)
        test_files = [test_files_dir + "/example_empty_file.yaml"]
        self.verify_empty_report(test_files_dir=None, files=test_files)

    def verify_empty_report(self, test_files_dir, files=None):
        runner_filter = RunnerFilter(framework=None, checks=None, skip_checks=None)
        runner_registry = RunnerRegistry(
            banner, runner_filter, tf_runner(), cfn_runner(), k8_runner()
        )
        reports = runner_registry.run(root_folder=test_files_dir, files=files)
        for report in reports:
            self.assertEqual(report.failed_checks, [])
            self.assertEqual(report.skipped_checks, [])
            self.assertEqual(report.passed_checks, [])
        return runner_registry

    def test_compact_json_output(self):
        test_files_dir = os.path.dirname(os.path.realpath(__file__)) + "/example_s3_tf"
        runner_filter = RunnerFilter(framework=None, checks=None, skip_checks=None)
        runner_registry = RunnerRegistry(
            banner, runner_filter, tf_runner(), cfn_runner(), k8_runner()
        )
        reports = runner_registry.run(root_folder=test_files_dir)

        config = argparse.Namespace(
            file=['./example_s3_tf/main.tf'],
            compact=True,
            output=['json'],
            quiet=False,
            soft_fail=False,
            soft_fail_on=None,
            hard_fail_on=None,
            output_file_path=None,
            use_enforcement_rules=None
        )

        with patch('sys.stdout', new=io.StringIO()) as captured_output:
            runner_registry.print_reports(scan_reports=reports, config=config)

        output = json.loads(captured_output.getvalue())
        passed_checks = output["results"]["passed_checks"]
        failed_checks = output["results"]["failed_checks"]

        assert all(check["code_block"] is None for check in passed_checks)
        assert all(check["connected_node"] is None for check in passed_checks)
        assert all(check["code_block"] is None for check in failed_checks)
        assert all(check["connected_node"] is None for check in failed_checks)

    def test_compact_csv_output(self):
        test_files_dir = os.path.dirname(os.path.realpath(__file__)) + "/example_s3_tf"
        runner_filter = RunnerFilter(framework=None, checks=None, skip_checks=None)
        runner_registry = RunnerRegistry(
            banner, runner_filter, tf_runner(), cfn_runner(), k8_runner()
        )
        reports = runner_registry.run(root_folder=test_files_dir)

        config = argparse.Namespace(
            file=['./example_s3_tf/main.tf'],
            compact=True,
            output=['csv'],
            quiet=False,
            soft_fail=False,
            soft_fail_on=None,
            hard_fail_on=None,
            output_file_path=None,
            use_enforcement_rules=None
        )

        with patch('sys.stdout', new=io.StringIO()) as captured_output:
            try:
                stream_handler = logging.StreamHandler(sys.stdout)
                logger.addHandler(stream_handler)

                runner_registry.print_reports(scan_reports=reports, config=config)
            finally:
                logger.removeHandler(stream_handler)

        output = captured_output.getvalue()

        self.assertIn('Persisting SBOM to ', output)
        iac_file_path = re.search("Persisting SBOM to (.*iac.csv)", output).group(1)
        with open(iac_file_path) as file:
            content = file.readlines()
            header = content[:1][0]
            self.assertEqual('Resource,Path,Git Org,Git Repository,Misconfigurations,Severity,Policy title,Guideline\n', header)
            rows = content[1:]
            self.assertIn('aws_s3_bucket', rows[0])
        oss_file_path = re.search("Persisting SBOM to (.*oss_packages.csv)", output).group(1)
        with open(oss_file_path) as file:
            content = file.readlines()
            header = content[:1][0]
            self.assertEqual('Package,Version,Path,Line(s),Git Org,Git Repository,Vulnerability,Severity,Description,Licenses,Fix Version,Registry URL,Root Package,Root Version\n', header)
            row = content[1:][0]
            self.assertIn('Prisma Cloud', row)

    def test_csv_invulnerable_report(self):
        report = Report('sca_package')
        report.extra_resources.add(
            ExtraResource(
                file_abs_path='/package.json',
                file_path='/package.json',
                resource='package.json.babel-jest',
                vulnerability_details={'package_name': 'babel-jest',
                                       'package_version': '',
                                       'licenses': 'Unknown',
                                       'package_type': ''},
            )
        )
        current_dir = os.path.dirname(os.path.realpath(__file__))
        results_path_dir = Path(current_dir) / 'packages_csv_results'
        if results_path_dir.exists() and results_path_dir.is_dir():
            shutil.rmtree(results_path_dir)

        os.mkdir(results_path_dir)
        config = argparse.Namespace(
            file=['./example_s3_tf/main.tf'],
            compact=False,
            output=['csv'],
            quiet=False,
            soft_fail=False,
            soft_fail_on=None,
            hard_fail_on=None,
            output_file_path=str(results_path_dir),
            use_enforcement_rules=None,
            bc_api_key='121223'
        )

        # when
        runner_filter = RunnerFilter(framework=None, checks=None, skip_checks=None)
        runner_registry = RunnerRegistry(banner, runner_filter)
        runner_registry.print_reports(scan_reports=[report], config=config)

        oss_packages_csv_file_name = ""
        for f in os.listdir(results_path_dir):
            if f.endswith('oss_packages.csv'):
                oss_packages_csv_file_name = f
                break

        assert oss_packages_csv_file_name

        oss_packages_csv = open(f'{results_path_dir}/{oss_packages_csv_file_name}')
        results = oss_packages_csv.read()
        expected_results = 'Package,Version,Path,Line(s),Git Org,Git Repository,Vulnerability,Severity,Description,Licenses,Fix Version,Registry URL,Root Package,Root Version' \
                           '\nbabel-jest,,/package.json,,,,,,,Unknown,N/A,,,\n'

        assert results == expected_results


    def test_run_with_empty_frameworks(self):
        # ensures that a run with a framework that gets filtered out (e.g. --framework terraform --file abc.yaml)
        # returns an empty report

        checkov_runners = [value for attr, value in CheckType.__dict__.items() if not attr.startswith("__")]
        scan_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'plan_with_hcl_for_enrichment', 'tfplan.json')

        runner_filter = RunnerFilter(framework=['terraform'], runners=checkov_runners)
        runner_registry = RunnerRegistry('', runner_filter, *DEFAULT_RUNNERS)
        runner_registry.filter_runners_for_files(['tfplan.json'])
        with self.assertLogs(level='ERROR') as log:
            reports = runner_registry.run(root_folder=None, files=[scan_file])
            self.assertEqual(len(reports), 0)  # checking that we get an empty report, not an exception
            self.assertIn(
                'There are no runners to run. This can happen if you specify a file type and a framework that are not compatible',
                ''.join(log.output))

    def test_runner_file_filter(self):
        checkov_runners = [value for attr, value in CheckType.__dict__.items() if not attr.startswith("__")]

        runner_filter = RunnerFilter(framework=['all'], runners=checkov_runners)
        runner_registry = RunnerRegistry(
            banner, runner_filter, *DEFAULT_RUNNERS
        )
        runner_registry.filter_runners_for_files([])
        self.assertEqual(set(runner_registry.runners), set(DEFAULT_RUNNERS))

        runner_filter = RunnerFilter(framework=['all'], runners=checkov_runners)
        runner_registry = RunnerRegistry(
            banner, runner_filter, *DEFAULT_RUNNERS
        )
        runner_registry.filter_runners_for_files(['main.tf'])
        self.assertEqual(set(r.check_type for r in runner_registry.runners), {'terraform', 'secrets'})

        runner_registry = RunnerRegistry(
            banner, runner_filter, *DEFAULT_RUNNERS, sca_package_runner_2()
        )
        runner_registry.filter_runners_for_files(['main.tf', 'requirements.txt'])
        self.assertEqual(set(r.check_type for r in runner_registry.runners), {'terraform', 'secrets', 'sca_package'})

        runner_filter = RunnerFilter(framework=['terraform'], runners=checkov_runners)
        runner_registry = RunnerRegistry(
            banner, runner_filter, *DEFAULT_RUNNERS
        )
        runner_registry.filter_runners_for_files(['main.tf'])
        self.assertEqual(set(r.check_type for r in runner_registry.runners), {'terraform'})

        runner_filter = RunnerFilter(framework=['all'], skip_framework=['secrets'], runners=checkov_runners)
        runner_registry = RunnerRegistry(
            banner, runner_filter, *DEFAULT_RUNNERS
        )
        runner_registry.filter_runners_for_files(['main.tf'])
        self.assertEqual(set(r.check_type for r in runner_registry.runners), {'terraform'})

        runner_filter = RunnerFilter(framework=['all'], skip_framework=['terraform'], runners=checkov_runners)
        runner_registry = RunnerRegistry(
            banner, runner_filter, *DEFAULT_RUNNERS
        )
        runner_registry.filter_runners_for_files(['main.tf'])
        self.assertEqual(set(r.check_type for r in runner_registry.runners), {'secrets'})

        runner_filter = RunnerFilter(framework=['all'], runners=checkov_runners)
        runner_registry = RunnerRegistry(
            banner, runner_filter, *DEFAULT_RUNNERS
        )
        runner_registry.filter_runners_for_files(['manifest.json'])
        self.assertIn("kubernetes", set(r.check_type for r in runner_registry.runners))

        runner_registry = RunnerRegistry(
            banner, runner_filter, *DEFAULT_RUNNERS, sca_package_runner_2()
        )
        runner_registry.filter_runners_for_files(['file.py'])
        self.assertEqual(set(r.check_type for r in runner_registry.runners), {'cdk', 'sast', 'secrets'})

    def test_runners_have_code_category(self):
        checkov_runners = [value for attr, value in CheckType.__dict__.items() if not attr.startswith("__")]
        for runner in checkov_runners:
            self.assertIn(runner, CodeCategoryMapping)

    def test_extract_git_info_from_account_id(self):
        account_id = "owner/name"
        expected_git_org = "owner"
        expected_git_repo = "name"
        result_git_org, result_git_repo = RunnerRegistry.extract_git_info_from_account_id(account_id)
        self.assertEqual(expected_git_repo, result_git_repo)
        self.assertEqual(expected_git_org, result_git_org)

        account_id = "owner/with/slash/separator/name"
        expected_git_org = "owner/with/slash/separator"
        expected_git_repo = "name"
        result_git_org, result_git_repo = RunnerRegistry.extract_git_info_from_account_id(account_id)
        self.assertEqual(expected_git_repo, result_git_repo)
        self.assertEqual(expected_git_org, result_git_org)

        account_id = "name"
        expected_git_org = ""
        expected_git_repo = ""
        result_git_org, result_git_repo = RunnerRegistry.extract_git_info_from_account_id(account_id)
        self.assertEqual(expected_git_repo, result_git_repo)
        self.assertEqual(expected_git_org, result_git_org)

        account_id = ""
        expected_git_org = ""
        expected_git_repo = ""
        result_git_org, result_git_repo = RunnerRegistry.extract_git_info_from_account_id(account_id)
        self.assertEqual(expected_git_repo, result_git_repo)
        self.assertEqual(expected_git_org, result_git_org)

    def test_merge_reports(self):
        # given
        runner_registry = RunnerRegistry(banner, RunnerFilter(), *DEFAULT_RUNNERS)
        reports = [
            [
                Report(check_type=CheckType.TERRAFORM),
                Report(check_type=CheckType.SCA_IMAGE),
            ],
            Report(check_type=CheckType.CLOUDFORMATION),
            Report(check_type=CheckType.SCA_IMAGE),
        ]

        # when
        merged_reports = runner_registry._merge_reports(reports=reports)

        # then
        merged_report_check_types = [
            report.check_type
            for report in merged_reports
        ]
        self.assertCountEqual(merged_report_check_types,[
            CheckType.TERRAFORM,
            CheckType.CLOUDFORMATION,
            CheckType.SCA_IMAGE,
        ])

    def test_merge_reports_for_multi_frameworks_image_referencer_results(self):
        # given
        runner_registry = RunnerRegistry(banner, RunnerFilter(), *DEFAULT_RUNNERS)
        tf_image_referencer_report = Report(check_type=CheckType.SCA_IMAGE)
        tf_image_referencer_report.image_cached_results = [
        {
            "dockerImageName": "busybox",
            "dockerFilePath": "/Users/arielk/dev/terragoat/terraform/aws/image-referencer.tf",
            "dockerFileContent": "image: busybox",
            "type": "Image",
            "sourceId": "ariel-cli/terragoat",
            "branch": "branch-name",
            "sourceType": "cli",
            "vulnerabilities":
            [
                {
                    "cveId": "CVE-2022-28391",
                    "status": "open",
                    "severity": "high",
                    "packageName": "busybox",
                    "packageVersion": "1.34.1",
                    "link": "https://nvd.nist.gov/vuln/detail/CVE-2022-28391",
                    "cvss": 8.8,
                    "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:R/S:U/C:H/I:H/A:H",
                    "description": "BusyBox through 1.35.0 allows remote attackers to execute arbitrary code if netstat is used to print a DNS PTR record\\'s value to a VT compatible terminal. Alternatively, the attacker could choose to change the terminal\\'s colors.",
                    "riskFactors":
                    [
                        "Attack complexity: low",
                        "Attack vector: network",
                        "High severity",
                        "Recent vulnerability",
                        "Remote execution"
                    ],
                    "publishedDate": "2022-04-03T21:15:00Z"
                }
            ],
            "packages":
            [],
            "relatedResourceId": "/Users/arielk/dev/terragoat/terraform/aws/image-referencer.tf:aws_batch_job_definition.test1111"
        }
        ]
        gha_image_referencer_report = Report(check_type=CheckType.SCA_IMAGE)
        gha_image_referencer_report.image_cached_results = [
        {
            "dockerImageName": "nginx:stable-alpine-perl",
            "dockerFilePath": "/.github/workflows/ci.yaml",
            "dockerFileContent": "image: nginx:stable-alpine-perl",
            "type": "Image",
            "sourceId": "arielkru/ak19-pr-sce-test",
            "branch": None,
            "sourceType": "Github",
            "vulnerabilities":
            [
                {
                    "cveId": "CVE-2020-35538",
                    "status": "open",
                    "severity": "medium",
                    "packageName": "libjpeg-turbo",
                    "packageVersion": "2.1.3-r1",
                    "link": "https://nvd.nist.gov/vuln/detail/CVE-2020-35538",
                    "cvss": 5.5,
                    "vector": "CVSS:3.1/AV:L/AC:L/PR:N/UI:R/S:U/C:N/I:N/A:H",
                    "description": "A crafted input file could cause a null pointer dereference in jcopy_sample_rows() when processed by libjpeg-turbo.",
                    "riskFactors":
                    [
                        "Attack complexity: low",
                        "Medium severity"
                    ],
                    "publishedDate": "2022-08-31T16:15:00Z"
                }
            ],
            "packages":
            [
                {
                    "type": "os",
                    "name": "tzdata",
                    "version": "2022a-r0",
                    "licenses":
                    [
                        "Public-Domain"
                    ]
                }
            ],
            "relatedResourceId": "jobs.container-test-job",
        }
        ]

        reports = [
            tf_image_referencer_report,
            gha_image_referencer_report
        ]

        # when
        merged_reports = runner_registry._merge_reports(reports=reports)

        # then
        assert len(merged_reports[0].image_cached_results) == 2

    def test_merge_reports_sca_supported_ir_report(self):
        from checkov.common.bridgecrew.platform_integration import bc_integration
        bc_integration.customer_run_config_response = {'supportedIrFw': 'terraform'}
        # given
        runner_registry = RunnerRegistry(banner, RunnerFilter(), *DEFAULT_RUNNERS)
        tf_report = Report(check_type=CheckType.TERRAFORM)
        tf_ir_report = Report(check_type=CheckType.SCA_IMAGE)
        tf_ir_report.image_cached_results = [
            {
                "dockerImageName": "busybox",
                "dockerFilePath": "/Users/arielk/dev/terragoat/terraform/aws/image-referencer.tf",
                "dockerFileContent": "image: busybox",
                "type": "Image",
                "sourceId": "ariel-cli/terragoat",
                "branch": "branch-name",
                "sourceType": "cli",
                "vulnerabilities":
                    [
                        {
                            "cveId": "CVE-2022-28391",
                            "status": "open",
                            "severity": "high",
                            "packageName": "busybox",
                            "packageVersion": "1.34.1",
                            "link": "https://nvd.nist.gov/vuln/detail/CVE-2022-28391",
                            "cvss": 8.8,
                            "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:R/S:U/C:H/I:H/A:H",
                            "description": "BusyBox through 1.35.0 allows remote attackers to execute arbitrary code if netstat is used to print a DNS PTR record\\'s value to a VT compatible terminal. Alternatively, the attacker could choose to change the terminal\\'s colors.",
                            "riskFactors":
                                [
                                    "Attack complexity: low",
                                    "Attack vector: network",
                                    "High severity",
                                    "Recent vulnerability",
                                    "Remote execution"
                                ],
                            "publishedDate": "2022-04-03T21:15:00Z"
                        }
                    ],
                "packages":
                    [],
                "relatedResourceId": "/Users/arielk/dev/terragoat/terraform/aws/image-referencer.tf:aws_batch_job_definition.test1111"
            }
        ]
        gha_report = Report(CheckType.GITHUB_ACTIONS)
        gha_ir_report = Report(check_type=CheckType.SCA_IMAGE)
        gha_ir_report.image_cached_results = [
            {
                "dockerImageName": "nginx:stable-alpine-perl",
                "dockerFilePath": "/.github/workflows/ci.yaml",
                "dockerFileContent": "image: nginx:stable-alpine-perl",
                "type": "Image",
                "sourceId": "arielkru/ak19-pr-sce-test",
                "branch": None,
                "sourceType": "Github",
                "vulnerabilities":
                    [
                        {
                            "cveId": "CVE-2020-35538",
                            "status": "open",
                            "severity": "medium",
                            "packageName": "libjpeg-turbo",
                            "packageVersion": "2.1.3-r1",
                            "link": "https://nvd.nist.gov/vuln/detail/CVE-2020-35538",
                            "cvss": 5.5,
                            "vector": "CVSS:3.1/AV:L/AC:L/PR:N/UI:R/S:U/C:N/I:N/A:H",
                            "description": "A crafted input file could cause a null pointer dereference in jcopy_sample_rows() when processed by libjpeg-turbo.",
                            "riskFactors":
                                [
                                    "Attack complexity: low",
                                    "Medium severity"
                                ],
                            "publishedDate": "2022-08-31T16:15:00Z"
                        }
                    ],
                "packages":
                    [
                        {
                            "type": "os",
                            "name": "tzdata",
                            "version": "2022a-r0",
                            "licenses":
                                [
                                    "Public-Domain"
                                ]
                        }
                    ],
                "relatedResourceId": "jobs.container-test-job",
            }
        ]
        sca_report = Report(CheckType.SCA_IMAGE)
        sca_report.image_cached_results = [
            {
                "dockerImageName": "nginx:stable-alpine-perl2",
                "dockerFilePath": "/.github/workflows/ci.yaml",
                "dockerFileContent": "image: nginx:stable-alpine-perl",
                "type": "Image",
                "sourceId": "arielkru/ak19-pr-sce-test",
                "branch": None,
                "sourceType": "Github",
                "vulnerabilities":
                    [
                        {
                            "cveId": "CVE-2020-35538",
                            "status": "open",
                            "severity": "medium",
                            "packageName": "libjpeg-turbo",
                            "packageVersion": "2.1.3-r1",
                            "link": "https://nvd.nist.gov/vuln/detail/CVE-2020-35538",
                            "cvss": 5.5,
                            "vector": "CVSS:3.1/AV:L/AC:L/PR:N/UI:R/S:U/C:N/I:N/A:H",
                            "description": "A crafted input file could cause a null pointer dereference in jcopy_sample_rows() when processed by libjpeg-turbo.",
                            "riskFactors":
                                [
                                    "Attack complexity: low",
                                    "Medium severity"
                                ],
                            "publishedDate": "2022-08-31T16:15:00Z"
                        }
                    ],
                "packages":
                    [
                        {
                            "type": "os",
                            "name": "tzdata",
                            "version": "2022a-r0",
                            "licenses":
                                [
                                    "Public-Domain"
                                ]
                        }
                    ],
                "relatedResourceId": "jobs.container-test-job",
            }
        ]

        reports = [
            [tf_report, tf_ir_report],
            [gha_report, gha_ir_report],
            sca_report
        ]

        # when
        merged_reports = runner_registry._merge_reports(reports=reports)

        # then
        sca_report = [r for r in merged_reports if r.check_type == CheckType.SCA_IMAGE]
        assert len(sca_report[0].image_cached_results) == 3
        assert len(runner_registry.sca_supported_ir_report.image_cached_results) == 2


def test_non_compact_json_output(capsys):
    # given
    test_files_dir = os.path.dirname(os.path.realpath(__file__)) + "/example_s3_tf"
    runner_filter = RunnerFilter(framework=None, checks=None, skip_checks=None)
    runner_registry = RunnerRegistry(
        banner, runner_filter, tf_runner(), cfn_runner(), k8_runner()
    )
    reports = runner_registry.run(root_folder=test_files_dir)

    config = argparse.Namespace(
        file=['./example_s3_tf/main.tf'],
        compact=False,
        output=['json'],
        quiet=False,
        soft_fail=False,
        soft_fail_on=None,
        hard_fail_on=None,
        output_file_path=None,
        use_enforcement_rules=None
    )

    # when
    runner_registry.print_reports(scan_reports=reports, config=config)

    # then
    captured = capsys.readouterr()

    assert 'code_block' in captured.out


def test_extra_resources_in_report(capsys):
    # given
    test_files_dir = os.path.dirname(os.path.realpath(__file__)) + "/example_bicep_with_empty_resources"
    runner_filter = RunnerFilter(framework=None, checks=None, skip_checks=None)
    runner_registry = RunnerRegistry(
        banner, runner_filter, bicep_runner()
    )
    reports = runner_registry.run(root_folder=test_files_dir)

    config = argparse.Namespace(
        file=['./example_bicep_with_empty_resources/playground.bicep'],
        compact=False,
        output=['json'],
        quiet=False,
        soft_fail=False,
        soft_fail_on=None,
        hard_fail_on=None,
        output_file_path=None,
        use_enforcement_rules=None
    )

    # when
    runner_registry.print_reports(scan_reports=reports, config=config)

    # then
    for report in reports:
        assert len(report.extra_resources) > 0


def test_extra_resources_removed_from_report(capsys):
    # given
    test_files_dir = os.path.dirname(os.path.realpath(__file__)) + "/example_bicep_with_empty_resources"
    runner_filter = RunnerFilter(framework=None, checks=None, skip_checks=None)
    runner_registry = RunnerRegistry(
        banner, runner_filter, bicep_runner()
    )
    reports = runner_registry.run(root_folder=test_files_dir)

    config = argparse.Namespace(
        file=['./example_bicep_with_empty_resources/playground.bicep'],
        compact=False,
        output=['json'],
        quiet=False,
        soft_fail=False,
        soft_fail_on=None,
        hard_fail_on=None,
        output_file_path=None,
        use_enforcement_rules=None,
        skip_resources_without_violations=True
    )

    # when
    runner_registry.print_reports(scan_reports=reports, config=config)

    # then
    for report in reports:
        assert len(report.extra_resources) == 0


def test_output_file_path_with_output_mapping(tmp_path: Path, capsys: CaptureFixture[str]):
    # given
    test_files_dir = Path(__file__).parent / "example_s3_tf"
    runner_filter = RunnerFilter(framework=None, checks=None, skip_checks=None)
    runner_registry = RunnerRegistry(
        banner, runner_filter, tf_runner(), cfn_runner(), k8_runner()
    )
    reports = runner_registry.run(root_folder=str(test_files_dir))

    json_file_path = tmp_path / "result.json"
    xml_file_path = tmp_path / "sub_folder/result.xml"
    config = argparse.Namespace(
        file=['./example_s3_tf/main.tf'],
        compact=False,
        output=["json", "cli", "junitxml"],
        quiet=False,
        soft_fail=False,
        soft_fail_on=None,
        hard_fail_on=None,
        output_file_path=f"{json_file_path},console,{xml_file_path}",
        use_enforcement_rules=None,
        output_bc_ids=False,
        summary_position="top",
    )

    # when
    runner_registry.print_reports(scan_reports=reports, config=config)

    # then
    assert 'By Prisma Cloud' in capsys.readouterr().out

    assert json_file_path.exists()
    assert '"check_type": "terraform"' in json_file_path.read_text()

    assert xml_file_path.exists()
    assert "<testcase " in xml_file_path.read_text()


def test_strip_code_blocks_from_json():
    # given
    reports = [
        {
            "check_type": "terraform",
            "results": {
                "passed_checks": [
                    {
                        "check_id": "CKV_AWS_88",
                        "bc_check_id": "BC_AWS_PUBLIC_12",
                        "check_name": "EC2 instance should not have public IP.",
                        "check_result": {
                            "result": CheckResult.PASSED,
                            "evaluated_keys": ["associate_public_ip_address"],
                        },
                        "code_block": [
                            (1, 'resource "aws_instance" "web_host" {\n'),
                            (2, "  # ec2 have plain text secrets in user data\n"),
                            (3, '  ami           = "${var.ami}"\n'),
                            (4, '  instance_type = "t2.nano"\n'),
                            (5, "\n"),
                            (6, "  vpc_security_group_ids = [\n"),
                            (7, '  "${aws_security_group.web-node.id}"]\n'),
                            (8, '  subnet_id = "${aws_subnet.web_subnet.id}"\n'),
                            (31, "}\n"),
                        ],
                        "file_path": "/ec2.tf",
                        "file_abs_path": "/ec2.tf",
                        "repo_file_path": "/ec2.tf",
                        "file_line_range": [1, 31],
                        "resource": "aws_instance.web_host",
                        "evaluations": None,
                        "check_class": "checkov.terraform.checks.resource.aws.EC2PublicIP",
                        "fixed_definition": None,
                        "entity_tags": {},
                        "caller_file_path": None,
                        "caller_file_line_range": None,
                        "resource_address": None,
                        "severity": None,
                        "bc_category": None,
                        "benchmarks": None,
                        "description": None,
                        "short_description": None,
                        "vulnerability_details": None,
                        "connected_node": None,
                        "guideline": "https://docs.bridgecrew.io/docs/public_12",
                        "details": [],
                        "check_len": None,
                        "definition_context_file_path": "/ec2.tf",
                    }
                ],
                "failed_checks": [],
                "skipped_checks": [],
                "parsing_errors": [
                    "/main.tf",
                ],
            },
            "summary": {
                "passed": 1,
                "failed": 0,
                "skipped": 0,
                "parsing_errors": 1,
                "resource_count": 0,
                "checkov_version": "2.2.330",
            },
            "url": "Add an api key '--bc-api-key <api-key>' to see more detailed insights via https://bridgecrew.cloud",
        }
    ]

    # when
    RunnerRegistry.strip_code_blocks_from_json(report_jsons=reports)

    # then
    assert reports[0]["results"]["passed_checks"][0]["code_block"] is None
    assert reports[0]["results"]["passed_checks"][0]["connected_node"] is None


def test_merge_runners():
    runner_cdk = CdkRunner()
    runner_sast = SastRunner()
    runner_tf = TFRunner()
    valid_runners = [(runner_cdk, None, None, None, None, None, None)]
    merged_runners = RunnerRegistry(None, None, None)._merge_runners(valid_runners)
    assert len(merged_runners) == 1
    assert isinstance(merged_runners[0][0], CdkRunner)

    valid_runners = [(runner_sast, None, None, None, None, None, None)]
    merged_runners = RunnerRegistry(None, None, None)._merge_runners(valid_runners)
    assert len(merged_runners) == 1
    assert isinstance(merged_runners[0][0], SastRunner)

    valid_runners = [(runner_cdk, None, None, None, None, None, None), (runner_sast, None, None, None, None, None, None)]
    merged_runners = RunnerRegistry(None, None, None)._merge_runners(valid_runners)
    assert len(merged_runners) == 1
    assert isinstance(merged_runners[0][0], SastRunner)
    assert len(merged_runners[0][0].cdk_langs) > 0

    valid_runners = [(runner_cdk, None, None, None, None, None, None), (runner_tf, None, None, None, None, None, None)]
    merged_runners = RunnerRegistry(None, None, None)._merge_runners(valid_runners)
    assert len(merged_runners) == 2
    assert isinstance(merged_runners[0][0], CdkRunner) or isinstance(merged_runners[0][0], TFRunner)
    assert isinstance(merged_runners[1][0], CdkRunner) or isinstance(merged_runners[1][0], TFRunner)


if __name__ == "__main__":
    unittest.main()



import os

from checkov.common.util.contextmanagers import temp_environ


def test_temp_environ():
    # given
    assert os.getenv("EXAMPLE_ENV_VAR") is None

    # when/then
    with temp_environ(EXAMPLE_ENV_VAR="example"):
        assert os.getenv("EXAMPLE_ENV_VAR") == "example"

    assert os.getenv("EXAMPLE_ENV_VAR") is None


def test_temp_environ_existing_env():
    # given
    os.environ["EXAMPLE_ENV_VAR"] = "example"

    # when/then
    with temp_environ(EXAMPLE_ENV_VAR="override_example"):
        assert os.getenv("EXAMPLE_ENV_VAR") == "override_example"

    assert os.environ["EXAMPLE_ENV_VAR"] == "example"
    del os.environ["EXAMPLE_ENV_VAR"]  # cleanup



import unittest
import hcl2

from checkov.terraform.checks.resource.aws.SubnetPublicIP import check
from checkov.common.models.enums import CheckResult


class TestSubnetPublicIP(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
                            resource "aws_subnet" "test" {
                              vpc_id     = aws_vpc.main.id
                              cidr_block = "10.0.1.0/24"

                              map_public_ip_on_launch = true
                            }
        """)
        resource_conf = hcl_res['resource'][0]['aws_subnet']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
                            resource "aws_subnet" "test" {
                              vpc_id     = aws_vpc.main.id
                              cidr_block = "10.0.1.0/24"
                              
                            }
        """)
        resource_conf = hcl_res['resource'][0]['aws_subnet']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_implicit(self):
        hcl_res = hcl2.loads("""
                            resource "aws_subnet" "test" {
                              vpc_id     = aws_vpc.main.id
                              cidr_block = "10.0.1.0/24"
                            
                              map_public_ip_on_launch = false
                            }
        """)
        resource_conf = hcl_res['resource'][0]['aws_subnet']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from pathlib import Path

import pytest

from checkov.bicep.runner import Runner
from checkov.arm.runner import Runner as ArmRunner
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.severities import Severities, BcSeverities
from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.graph.db_connectors.rustworkx.rustworkx_db_connector import RustworkxConnector
from checkov.runner_filter import RunnerFilter
from checkov.bicep.checks.resource.registry import registry as resource_registry
from checkov.bicep.checks.param.registry import registry as param_registry

EXAMPLES_DIR = Path(__file__).parent / "examples"


@pytest.fixture(autouse=True)
def load_arm_checks():
    # just initialize to add the ARM checks to the Bicep registry
    ArmRunner()


def test_registry_has_type():
    assert resource_registry.report_type == CheckType.BICEP
    assert param_registry.report_type == CheckType.BICEP


def test_arm_checks_laoded():
    # when
    resource_registry = Runner.block_type_registries["resources"]

    # then
    assert len(resource_registry.checks) > 30


@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector
    ]
)
def test_runner_passing_check(graph_connector):
    # given
    test_file = EXAMPLES_DIR / "playground.bicep"

    # when
    report = Runner(db_connector=graph_connector()).run(root_folder="", files=[str(test_file)], runner_filter=RunnerFilter(checks=["CKV_AZURE_3"]))

    # then
    summary = report.get_summary()

    assert summary["passed"] == 1
    assert summary["failed"] == 0
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0

@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector
    ]
)
def test_runner_failing_check(graph_connector):
    # given
    test_file = EXAMPLES_DIR / "playground.bicep"

    # when
    report = Runner(db_connector=graph_connector()).run(root_folder="", files=[str(test_file)], runner_filter=RunnerFilter(checks=["CKV_AZURE_9"]))

    # then
    summary = report.get_summary()

    assert summary["passed"] == 1
    assert summary["failed"] == 1
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0

@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector
    ]
)
def test_runner_skipping_check(graph_connector):
    # given
    test_file = EXAMPLES_DIR / "playground.bicep"

    # when
    report = Runner(db_connector=graph_connector()).run(root_folder="", files=[str(test_file)], runner_filter=RunnerFilter(checks=["CKV_AZURE_35"]))

    # then
    summary = report.get_summary()

    assert summary["passed"] == 0
    assert summary["failed"] == 0
    assert summary["skipped"] == 1
    assert summary["parsing_errors"] == 0

@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector
    ]
)
def test_runner_honors_enforcement_rules(graph_connector):
    # given
    test_files = list(map(lambda f: str(f), [EXAMPLES_DIR / "playground.bicep", EXAMPLES_DIR / "graph.bicep"]))

    # when
    filter = RunnerFilter(framework=['bicep'], use_enforcement_rules=True)
    # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
    # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
    filter.enforcement_rule_configs = {CheckType.BICEP: Severities[BcSeverities.OFF]}
    report = Runner(db_connector=graph_connector()).run(root_folder="", files=test_files, runner_filter=filter)

    # then
    summary = report.get_summary()

    assert summary["passed"] == 0
    assert summary["failed"] == 0
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0

@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector
    ]
)
def test_runner_parsing_errors(graph_connector):
    # given
    test_file = EXAMPLES_DIR / "malformed.bicep"

    # when
    report = Runner(db_connector=graph_connector()).run(root_folder="", files=[str(test_file)], runner_filter=RunnerFilter(checks=["CKV_AZURE_35"]))

    # then
    summary = report.get_summary()

    assert summary["passed"] == 0
    assert summary["failed"] == 0
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 1

@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector
    ]
)
def test_runner_ignore_existing_resource(graph_connector):
    # given
    test_file = EXAMPLES_DIR / "existing.bicep"

    # when
    report = Runner(db_connector=graph_connector()).run(root_folder="", files=[str(test_file)], runner_filter=RunnerFilter(checks=["CKV_AZURE_35"]))

    # then
    summary = report.get_summary()

    assert summary["passed"] == 0
    assert summary["failed"] == 1
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0
    assert summary["resource_count"] == 2  # 1 should be unknown

    assert report.failed_checks[0].resource == "Microsoft.Storage/storageAccounts.storageAccount"

@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector
    ]
)
def test_runner_extra_resources(graph_connector):
    # given
    test_file = EXAMPLES_DIR / "playground.bicep"

    # when
    report = Runner(db_connector=graph_connector()).run(root_folder="", files=[str(test_file)], runner_filter=RunnerFilter(checks=["CKV_AZURE_3"]))

    # then
    summary = report.get_summary()

    assert summary["passed"] == 1

    assert len(report.extra_resources) == 7
    extra_resource = next(
        resource for resource in report.extra_resources if resource.resource == "Microsoft.Compute/virtualMachines.vm"
    )
    assert extra_resource.file_abs_path == str(test_file)
    assert extra_resource.file_path.endswith("playground.bicep")


def test_runner_loop_resource():
    # given
    test_file = EXAMPLES_DIR / "loop.bicep"

    # when
    report = Runner().run(root_folder="", files=[str(test_file)], runner_filter=RunnerFilter(checks=["CKV_AZURE_2"]))

    # then
    summary = report.get_summary()

    assert summary["passed"] == 1
    assert summary["failed"] == 0
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0



from aws_cdk import core
from aws_cdk import aws_glue as glue

class MyGlueDataCatalogEncryptionSettingsStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define Data Catalog encryption settings
        data_catalog_encryption_settings = glue.CfnDataCatalogEncryptionSettings(
            self, 'MyGlueDataCatalogEncryptionSettings',
            data_catalog_encryption_settings={
                'ConnectionPasswordEncryption': {
                    'ReturnConnectionPasswordEncrypted': True
                },
                'EncryptionAtRest': {
                    'CatalogEncryptionMode': 'SSE-KMS'
                }
            }
        )

app = core.App()
MyGlueDataCatalogEncryptionSettingsStack(app, "MyGlueDataCatalogEncryptionSettingsStack")
app.synth()



from __future__ import annotations

import unittest
import json
from typing import Any

import jsonschema
import urllib.request

from checkov.common.models.enums import CheckResult
from checkov.common.output.report import Report
from checkov.common.output.record import Record
from checkov.common.output.sarif import Sarif


class TestSarifReport(unittest.TestCase):
    def test_valid_passing_valid_testcases(self):
        # given
        record1 = get_ckv_aws_21_record()
        record1.set_guideline("https://docs.bridgecrew.io/docs/s3_16-enable-versioning")

        record2 = Record(
            check_id="CKV_AWS_3",
            check_name="Ensure all data stored in the EBS is securely encrypted",
            check_result={"result": CheckResult.FAILED},
            code_block=[
                (5, 'resource aws_ebs_volume "web_host_storage" {\n'),
                (6, '  availability_zone = "us-west-2a"\n'),
                (7, "}\n"),
            ],
            file_path="./ec2.tf",
            file_line_range=[5, 7],
            resource="aws_ebs_volume.web_host_storage",
            evaluations=None,
            check_class=None,
            file_abs_path="/path to/ec2.tf",  # spaces should be handled correctly
            entity_tags={"tag1": "value1"},
        )
        record2.set_guideline("https://docs.bridgecrew.io/docs/general_7")

        r = Report("terraform")
        r.add_record(record=record1)
        r.add_record(record=record2)

        #  when
        sarif = Sarif(reports=[r], tool="")

        # then
        self.assertEqual(
            None,
            jsonschema.validate(instance=sarif.json, schema=get_sarif_schema()),
        )

        sarif.json["runs"][0]["tool"]["driver"]["version"] = "9.9.9"  # override the version

        self.assertDictEqual(
            sarif.json,
            {
                "$schema": "https://raw.githubusercontent.com/oasis-tcs/sarif-spec/main/sarif-2.1/schema/sarif-schema-2.1.0.json",
                "version": "2.1.0",
                "runs": [
                    {
                        "tool": {
                            "driver": {
                                "name": "Bridgecrew",
                                "version": "9.9.9",
                                "informationUri": "https://docs.bridgecrew.io",
                                "rules": [
                                    {
                                        "id": "CKV_AWS_21",
                                        "name": "Some Check",
                                        "shortDescription": {"text": "Some Check"},
                                        "fullDescription": {"text": "Some Check"},
                                        "help": {"text": "Some Check\nResource: aws_s3_bucket.operations"},
                                        "helpUri": "https://docs.bridgecrew.io/docs/s3_16-enable-versioning",
                                        "defaultConfiguration": {"level": "error"},
                                    },
                                    {
                                        "id": "CKV_AWS_3",
                                        "name": "Ensure all data stored in the EBS is securely encrypted",
                                        "shortDescription": {
                                            "text": "Ensure all data stored in the EBS is securely encrypted"
                                        },
                                        "fullDescription": {
                                            "text": "Ensure all data stored in the EBS is securely encrypted"
                                        },
                                        "help": {
                                            "text": "Ensure all data stored in the EBS is securely encrypted\nResource: aws_ebs_volume.web_host_storage"
                                        },
                                        "helpUri": "https://docs.bridgecrew.io/docs/general_7",
                                        "defaultConfiguration": {"level": "error"},
                                    },
                                ],
                                "organization": "bridgecrew",
                            }
                        },
                        "results": [
                            {
                                "ruleId": "CKV_AWS_21",
                                "ruleIndex": 0,
                                "level": "error",
                                "attachments": [],
                                "message": {"text": "Some Check"},
                                "locations": [
                                    {
                                        "physicalLocation": {
                                            "artifactLocation": {"uri": "s3.tf"},
                                            "region": {
                                                "startLine": 1,
                                                "endLine": 3,
                                                "snippet": {
                                                    "text": 'resource aws_s3_bucket "operations" {\n  bucket = "example"\n}\n'
                                                },
                                            },
                                        }
                                    }
                                ],
                            },
                            {
                                "ruleId": "CKV_AWS_3",
                                "ruleIndex": 1,
                                "level": "error",
                                "attachments": [],
                                "message": {"text": "Ensure all data stored in the EBS is securely encrypted"},
                                "locations": [
                                    {
                                        "physicalLocation": {
                                            "artifactLocation": {"uri": "path%20to/ec2.tf"},
                                            "region": {
                                                "startLine": 5,
                                                "endLine": 7,
                                                "snippet": {
                                                    "text": 'resource aws_ebs_volume "web_host_storage" {\n  availability_zone = "us-west-2a"\n}\n'
                                                },
                                            },
                                        }
                                    }
                                ],
                            },
                        ],
                    }
                ],
            },
        )

    def test_multiple_instances_of_same_rule_do_not_break_schema(self):
        record1 = get_ckv_aws_21_record()
        record1.set_guideline("")

        record2 = Record(
            check_id="CKV_AWS_111",
            check_name="Ensure IAM policies does not allow write access without constraints",
            check_result={"result": CheckResult.FAILED},
            code_block=[(1, "some code")],
            file_path="./ec2.tf",
            file_line_range=[22, 25],
            resource="aws_ebs_volume.web_host_storage",
            evaluations=None,
            check_class=None,
            file_abs_path=",.",
            entity_tags={"tag1": "value1"},
        )
        record2.set_guideline("")

        record3 = Record(
            check_id="CKV2_AWS_3",
            check_name="Ensure GuardDuty is enabled to specific org/region",
            check_result={"result": CheckResult.FAILED},
            code_block=[(1, "some code")],
            file_path="./ec2.tf",
            file_line_range=[1, 3],
            resource="aws_ebs_volume.web_host_storage",
            evaluations=None,
            check_class=None,
            file_abs_path=",.",
            entity_tags={"tag1": "value1"},
        )
        record3.set_guideline("")

        record4 = Record(
            check_id="CKV2_AWS_3",
            check_name="Ensure GuardDuty is enabled to specific org/region",
            check_result={"result": CheckResult.FAILED},
            code_block=[(1, "some code")],
            file_path="./org.tf",
            file_line_range=[7, 10],
            resource="aws_ebs_volume.web_host_storage",
            evaluations=None,
            check_class=None,
            file_abs_path=",.",
            entity_tags={"tag1": "value1"},
        )
        record4.set_guideline("")

        record5 = Record(
            check_id="CKV2_AWS_3",
            check_name="Ensure GuardDuty is enabled to specific org/region",
            check_result={"result": CheckResult.FAILED},
            code_block=[(1, "some code")],
            file_path="./org.tf",
            file_line_range=[15, 20],
            resource="aws_ebs_volume.web_host_storage",
            evaluations=None,
            check_class=None,
            file_abs_path=",.",
            entity_tags={"tag1": "value1"},
        )
        record5.set_guideline("")

        record6 = Record(
            check_id="CKV2_AWS_3",
            check_name="Ensure GuardDuty is enabled to specific org/region",
            check_result={"result": CheckResult.FAILED},
            code_block=[(1, "some code")],
            file_path="./org.tf",
            file_line_range=[25, 28],
            resource="aws_ebs_volume.web_host_storage",
            evaluations=None,
            check_class=None,
            file_abs_path=",.",
            entity_tags={"tag1": "value1"},
        )
        record6.set_guideline("")

        record7 = Record(
            check_id="CKV_AWS_107",
            check_name="Ensure IAM policies does not allow credentials exposure",
            check_result={"result": CheckResult.FAILED},
            code_block=[(1, "some code")],
            file_path="./ec2.tf",
            file_line_range=[30, 35],
            resource="aws_ebs_volume.web_host_storage",
            evaluations=None,
            check_class=None,
            file_abs_path=",.",
            entity_tags={"tag1": "value1"},
        )
        record7.set_guideline("")

        record8 = Record(
            check_id="CKV_AWS_110",
            check_name="Ensure IAM policies does not allow privilege escalation",
            check_result={"result": CheckResult.FAILED},
            code_block=[(1, "some code")],
            file_path="./ec2.tf",
            file_line_range=[30, 35],
            resource="aws_ebs_volume.web_host_storage",
            evaluations=None,
            check_class=None,
            file_abs_path=",.",
            entity_tags={"tag1": "value1"},
        )
        record8.set_guideline("")

        record9 = Record(
            check_id="CKV_AWS_110",
            check_name="Ensure IAM policies does not allow privilege escalation",
            check_result={"result": CheckResult.FAILED},
            code_block=[(1, "some code")],
            file_path="./ec2.tf",
            file_line_range=[38, 40],
            resource="aws_ebs_volume.web_host_storage",
            evaluations=None,
            check_class=None,
            file_abs_path=",.",
            entity_tags={"tag1": "value1"},
        )
        record9.set_guideline("")

        # Record with non-empty guideline
        record10 = Record(
            check_id="CKV_AWS_23",
            check_name="Some Check",
            check_result={"result": CheckResult.FAILED},
            code_block=[(1, "some code")],
            file_path="./s3.tf",
            file_line_range=[1, 3],
            resource="aws_s3_bucket.operations",
            evaluations=None,
            check_class=None,
            file_abs_path=",.",
            entity_tags={"tag1": "value1"},
        )
        record10.set_guideline("https://example.com")

        # Record without guideline
        record11 = Record(
            check_id="CKV_AWS_24",
            check_name="Some Check",
            check_result={"result": CheckResult.FAILED},
            code_block=[(1, "some code")],
            file_path="./s3.tf",
            file_line_range=[1, 3],
            resource="aws_s3_bucket.operations",
            evaluations=None,
            check_class=None,
            file_abs_path=",.",
            entity_tags={"tag1": "value1"},
        )
        # No guideline here

        r = Report("terraform")
        r.add_record(record=record1)
        r.add_record(record=record2)
        r.add_record(record=record3)
        r.add_record(record=record4)
        r.add_record(record=record5)
        r.add_record(record=record6)
        r.add_record(record=record7)
        r.add_record(record=record8)
        r.add_record(record=record9)
        r.add_record(record=record10)
        r.add_record(record=record11)

        sarif = Sarif(reports=[r], tool="")

        self.assertEqual(
            None,
            jsonschema.validate(instance=sarif.json, schema=get_sarif_schema()),
        )
        self.assertFalse(are_duplicates_in_sarif_rules(sarif.json))
        self.assertTrue(are_rule_indexes_correct_in_results(sarif.json))
        self.assertTrue(are_rules_without_help_uri_correct(sarif.json))

    def test_non_url_guideline_link(self):
        # given
        record1 = get_ckv_aws_21_record()
        record1.set_guideline("some random text")

        r = Report("terraform")
        r.add_record(record=record1)

        #  when
        sarif = Sarif(reports=[r], tool="")

        # then
        self.assertEqual(
            None,
            jsonschema.validate(instance=sarif.json, schema=get_sarif_schema()),
        )

        sarif.json["runs"][0]["tool"]["driver"]["version"] = "9.9.9"  # override the version

        # sarif.json["runs"][0]["tool"]["driver"]["rules"][0] shouldn't include key "helpUri"
        self.assertDictEqual(
            sarif.json,
            {
                "$schema": "https://raw.githubusercontent.com/oasis-tcs/sarif-spec/main/sarif-2.1/schema/sarif-schema-2.1.0.json",
                "version": "2.1.0",
                "runs": [
                    {
                        "tool": {
                            "driver": {
                                "name": "Bridgecrew",
                                "version": "9.9.9",
                                "informationUri": "https://docs.bridgecrew.io",
                                "rules": [
                                    {
                                        "id": "CKV_AWS_21",
                                        "name": "Some Check",
                                        "shortDescription": {"text": "Some Check"},
                                        "fullDescription": {"text": "Some Check"},
                                        "help": {"text": "Some Check\nResource: aws_s3_bucket.operations"},
                                        "defaultConfiguration": {"level": "error"},
                                    }
                                ],
                                "organization": "bridgecrew",
                            }
                        },
                        "results": [
                            {
                                "ruleId": "CKV_AWS_21",
                                "ruleIndex": 0,
                                "level": "error",
                                "attachments": [],
                                "message": {"text": "Some Check"},
                                "locations": [
                                    {
                                        "physicalLocation": {
                                            "artifactLocation": {"uri": "s3.tf"},
                                            "region": {
                                                "startLine": 1,
                                                "endLine": 3,
                                                "snippet": {
                                                    "text": 'resource aws_s3_bucket "operations" {\n  bucket = "example"\n}\n'
                                                },
                                            },
                                        }
                                    }
                                ],
                            }
                        ],
                    }
                ],
            },
        )


def get_sarif_schema() -> dict[str, Any]:
    file_name, headers = urllib.request.urlretrieve(
        "https://raw.githubusercontent.com/oasis-tcs/sarif-spec/main/sarif-2.1/schema/sarif-schema-2.1.0.json"
    )
    with open(file_name, "r") as file:
        schema = json.load(file)
    return schema


def get_ckv_aws_21_record() -> Record:
    return Record(
        check_id="CKV_AWS_21",
        check_name="Some Check",
        check_result={"result": CheckResult.FAILED},
        code_block=[
            (1, 'resource aws_s3_bucket "operations" {\n'),
            (2, '  bucket = "example"\n'),
            (3, "}\n"),
        ],
        file_path="./s3.tf",
        file_line_range=[1, 3],
        resource="aws_s3_bucket.operations",
        evaluations=None,
        check_class=None,
        file_abs_path="./s3.tf",
        entity_tags={"tag1": "value1"},
    )

def are_duplicates_in_sarif_rules(sarif_json) -> bool:
    rules = sarif_json["runs"][0]["tool"]["driver"]["rules"]
    ruleset = set()
    for rule in rules:
        ruleset.add(rule["id"])

    return len(rules) != len(ruleset)


def are_rule_indexes_correct_in_results(sarif_json) -> bool:
    rules = sarif_json["runs"][0]["tool"]["driver"]["rules"]
    results = sarif_json["runs"][0]["results"]
    for rule in rules:
        for result in results:
            if result["ruleId"] == rule["id"]:
                if result["ruleIndex"] != rules.index(rule) or result["ruleIndex"] > len(rules):
                    return False
    return True


def are_rules_without_help_uri_correct(sarif_json) -> bool:
    rules = sarif_json["runs"][0]["tool"]["driver"]["rules"]
    results = sarif_json["runs"][0]["results"]
    for rule in rules:
        if "helpUri" in rule:
            if rule["helpUri"] is None or rule["helpUri"] == "":
                return False
    return True


if __name__ == "__main__":
    unittest.main()



from aws_cdk import core
from aws_cdk import aws_glue as glue

class MyGlueSecurityConfigurationStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define the security configuration with encryption settings
        security_configuration = glue.CfnSecurityConfiguration(
            self, 'MyGlueSecurityConfiguration',
            encryption_configuration={
                'CloudWatchEncryption': {
                    'CloudWatchEncryptionMode': 'SSE-KMS'
                },
                'JobBookmarksEncryption': {
                    'JobBookmarksEncryptionMode': 'DISABLED'
                },
                'S3Encryptions': [
                    {
                        'S3EncryptionMode': 'SSE-KMS'
                    }
                ]
            }
        )

app = core.App()
MyGlueSecurityConfigurationStack(app, "MyGlueSecurityConfigurationStack")
app.synth()

class MyGlueSecurityConfigurationStack2(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define the security configuration with encryption settings
        security_configuration = glue.CfnSecurityConfiguration(
            self, 'MyGlueSecurityConfiguration',
            encryption_configuration={
                'JobBookmarksEncryption': {
                    'JobBookmarksEncryptionMode': 'CSE-KMS'
                },
                'S3Encryptions': [
                    {
                        'S3EncryptionMode': 'SSE-KMS'
                    }
                ]
            }
        )

app = core.App()
MyGlueSecurityConfigurationStack2(app, "MyGlueSecurityConfigurationStack2")
app.synth()



import os
from pathlib import Path
from unittest.mock import MagicMock

from pytest_mock import MockerFixture
from packaging import version as packaging_version

from checkov.common.bridgecrew.bc_source import SourceTypes, BCSourceType
from checkov.common.bridgecrew.code_categories import CodeCategoryType
from checkov.common.bridgecrew.platform_integration import bc_integration, FileToPersist
from checkov.runner_filter import RunnerFilter
from checkov.sca_package_2.runner import Runner
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.severities import Severities, BcSeverities
from checkov.common.models.enums import CheckResult, ErrorStatus

EXAMPLES_DIR = Path(__file__).parent / "examples"


def test_upload_scannable_files():
    # when
    bc_integration.bc_api_key = "abcd1234-abcd-1234-abcd-1234abcd1234"
    input_paths = Runner().upload_package_files(
        root_path=EXAMPLES_DIR,
        files=[],
        excluded_paths=set(),
    )

    # expected
    expected_output = {
        FileToPersist(full_file_path=str(EXAMPLES_DIR / 'requirements.txt'),
                      s3_file_key='requirements.txt'),
        FileToPersist(full_file_path=str(EXAMPLES_DIR / 'go.sum'),
                      s3_file_key='go.sum'),
        FileToPersist(full_file_path=str(EXAMPLES_DIR / 'package-lock.json'),
                      s3_file_key='package-lock.json'),
        FileToPersist(full_file_path=str(EXAMPLES_DIR / 'package.json'),
                      s3_file_key='package.json'),
        FileToPersist(full_file_path=str(EXAMPLES_DIR / 'go.mod'),
                      s3_file_key='go.mod'),
        FileToPersist(full_file_path=str(EXAMPLES_DIR / 'Microsoft.NET.Sdk.csproj'),
                      s3_file_key='Microsoft.NET.Sdk.csproj')
    }

    # then
    assert len(input_paths) == 6

    assert set(input_paths) == expected_output


def test_upload_scannable_files_exclude_go_and_requirements():
    # when
    input_output_paths = Runner().upload_package_files(
        root_path=EXAMPLES_DIR,
        files=[],
        excluded_paths=set(),
        excluded_file_names={"go.sum", "package-lock.json"}
    )
    # expected
    expected_output = {
        FileToPersist(full_file_path=str(EXAMPLES_DIR / 'requirements.txt'),
                      s3_file_key='requirements.txt'),
        FileToPersist(full_file_path=str(EXAMPLES_DIR / 'package.json'),
                      s3_file_key='package.json'),
        FileToPersist(full_file_path=str(EXAMPLES_DIR / 'go.mod'),
                      s3_file_key='go.mod'),
        FileToPersist(full_file_path=str(EXAMPLES_DIR / 'Microsoft.NET.Sdk.csproj'),
                      s3_file_key='Microsoft.NET.Sdk.csproj')
    }

    # then
    assert len(input_output_paths) == 4

    assert set(input_output_paths) == expected_output


def test_upload_scannable_files_file_config():
    origin_cwd = os.getcwd()
    try:
        # setup
        os.chdir(str(Path(__file__).parent))

        # when
        input_output_paths = Runner().upload_package_files(
            root_path=None,
            files=[
                str(EXAMPLES_DIR / 'requirements.txt'),
                str(EXAMPLES_DIR / 'go.sum'),
                str(EXAMPLES_DIR / 'package-lock.json'),
                str(EXAMPLES_DIR / 'package.json'),
                str(EXAMPLES_DIR / 'go.mod'),
                str(EXAMPLES_DIR / 'Microsoft.NET.Sdk.csproj')
            ],
            excluded_paths=set(),
            excluded_file_names=set()
        )
        # expected (paths are in related to the test-working-dir)
        expected_output = {
            FileToPersist(full_file_path=str(EXAMPLES_DIR / 'requirements.txt'),
                          s3_file_key='examples/requirements.txt'),
            FileToPersist(full_file_path=str(EXAMPLES_DIR / 'go.sum'),
                          s3_file_key='examples/go.sum'),
            FileToPersist(full_file_path=str(EXAMPLES_DIR / 'package-lock.json'),
                          s3_file_key='examples/package-lock.json'),
            FileToPersist(full_file_path=str(EXAMPLES_DIR / 'package.json'),
                          s3_file_key='examples/package.json'),
            FileToPersist(full_file_path=str(EXAMPLES_DIR / 'go.mod'),
                          s3_file_key='examples/go.mod'),
            FileToPersist(full_file_path=str(EXAMPLES_DIR / 'Microsoft.NET.Sdk.csproj'),
                          s3_file_key='examples/Microsoft.NET.Sdk.csproj')
        }

        # then
        assert len(input_output_paths) == 6

        assert set(input_output_paths) == expected_output
    finally:
        # teardown
        os.chdir(origin_cwd)


def test_run(sca_package_2_report):
    # given
    report = sca_package_2_report
    # then
    assert report.check_type == "sca_package"

    assert report.resources == {
        "path/to/go.sum.github.com/dgrijalva/jwt-go",
        "path/to/go.sum.golang.org/x/crypto",
        "requirements.txt.django",
        "requirements.txt.flask",
        "requirements.txt.requests",
        "path/to/sub/requirements.txt.requests",
    }

    assert len(report.passed_checks) == 3
    assert len(report.failed_checks) == 7
    assert len(report.skipped_checks) == 3
    assert len(report.parsing_errors) == 0

    cve_record = next((c for c in report.failed_checks if
                       c.resource == "path/to/go.sum.golang.org/x/crypto" and c.check_name == "SCA package scan"), None)
    assert cve_record is not None
    assert cve_record.bc_check_id == "BC_CVE_2020_29652"
    assert cve_record.check_id == "CKV_CVE_2020_29652"
    assert cve_record.check_class == "unittest.mock.MagicMock"  # not the real one
    assert cve_record.check_name == "SCA package scan"
    assert cve_record.check_result == {"result": CheckResult.FAILED}
    assert cve_record.code_block == [(0, "golang.org/x/crypto: v0.0.1")]
    assert cve_record.description == (
        "A nil pointer dereference in the golang.org/x/crypto/ssh component through v0.0.3 "
        "for Go allows remote attackers to cause a denial of service against SSH servers."
    )
    assert cve_record.file_abs_path == "/path/to/go.sum"
    assert cve_record.file_line_range == [0, 0]
    assert cve_record.file_path == "/path/to/go.sum"
    assert cve_record.repo_file_path == "/path/to/go.sum"
    assert cve_record.resource == "path/to/go.sum.golang.org/x/crypto"
    assert cve_record.severity == Severities[BcSeverities.HIGH]
    assert cve_record.short_description == "CVE-2020-29652 - golang.org/x/crypto: v0.0.1"
    assert cve_record.vulnerability_details["lowest_fixed_version"] == "0.0.2"
    assert cve_record.vulnerability_details["fixed_versions"] == [
        packaging_version.parse("v0.0.2"),
    ]

    # making sure cve-records have licenses (the one belongs to the associated package) - this data will be printed
    # as part of the BON report.
    cve_record_with_license = next((c for c in report.failed_checks if
                                    c.resource == "requirements.txt.django" and c.check_name == "SCA package scan"),
                                   None)
    assert cve_record_with_license is not None
    assert "licenses" in cve_record_with_license.vulnerability_details
    assert cve_record_with_license.vulnerability_details["licenses"] == "OSI_BDS"

    cve_record_with_2_license = next((c for c in report.failed_checks if
                                      c.resource == "requirements.txt.flask" and c.check_name == "SCA package scan"),
                                     None)
    assert cve_record_with_2_license is not None
    assert "licenses" in cve_record_with_2_license.vulnerability_details
    assert cve_record_with_2_license.vulnerability_details["licenses"] == '"OSI_APACHE","DUMMY_OTHER_LICENSE"'

    # making sure extra-resources (a scanned packages without cves) also have licenses - this data will be printed
    # as part of the BON report.
    extra_resource = next((c for c in report.extra_resources if c.resource == "requirements.txt.requests"),
                          None)
    assert extra_resource is not None
    assert "licenses" in extra_resource.vulnerability_details
    assert extra_resource.vulnerability_details["licenses"] == "OSI_APACHE"

    license_resource = next((c for c in report.failed_checks if c.check_name == "SCA license" if
                             c.resource == "requirements.txt.flask"), None)
    assert license_resource is not None
    assert license_resource.check_id == "BC_LIC_1"
    assert license_resource.bc_check_id == "BC_LIC_1"
    assert license_resource.check_result == {"result": CheckResult.FAILED}
    assert {"package_name", "package_name", "license", "status",
            "policy"} <= license_resource.vulnerability_details.keys()
    assert license_resource.vulnerability_details["package_name"] == "flask"
    assert license_resource.vulnerability_details["package_version"] == "0.6"
    assert license_resource.vulnerability_details["license"] == "DUMMY_OTHER_LICENSE"
    assert license_resource.vulnerability_details["status"] == "FAILED"
    assert license_resource.vulnerability_details["policy"] == "BC_LIC_1"


def test_runner_honors_enforcement_rules(mocker: MockerFixture, scan_result_2):
    # given
    bc_integration.bc_api_key = "abcd1234-abcd-1234-abcd-1234abcd1234"
    scanner_mock = MagicMock()
    scanner_mock.return_value.scan.return_value = scan_result_2
    mocker.patch("checkov.sca_package_2.runner.Scanner", side_effect=scanner_mock)

    # when
    runner = Runner()
    filter = RunnerFilter(framework=['sca_package'], use_enforcement_rules=True)
    # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
    # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
    filter.enforcement_rule_configs = {
        CheckType.SCA_PACKAGE: {
            CodeCategoryType.LICENSES: Severities[BcSeverities.OFF],
            CodeCategoryType.VULNERABILITIES: Severities[BcSeverities.OFF]
        }
    }
    report = runner.run(root_folder=EXAMPLES_DIR, runner_filter=filter)

    # then
    summary = report.get_summary()
    # then
    assert summary["passed"] == 0
    assert summary["failed"] == 0
    assert summary["skipped"] > 0

    assert any(c for c in report.skipped_checks if c.check_id.startswith('CKV_CVE'))
    assert any(c for c in report.skipped_checks if c.check_id.startswith('BC_LIC'))

    assert summary["parsing_errors"] == 0


def test_run_license_policy(mocker: MockerFixture, scan_result_2):
    # given
    bc_integration.bc_api_key = "abcd1234-abcd-1234-abcd-1234abcd1234"
    scanner_mock = MagicMock()
    scanner_mock.return_value.scan.return_value = scan_result_2
    mocker.patch("checkov.sca_package_2.runner.Scanner", side_effect=scanner_mock)

    # when
    runner = Runner()
    filter = RunnerFilter(framework=['sca_package'], checks=['BC_LIC_1'])
    report = runner.run(root_folder=EXAMPLES_DIR, runner_filter=filter)

    # then
    assert not [c for c in report.passed_checks + report.failed_checks if c.check_id.startswith('CKV_CVE')]


def test_run_with_empty_scan_result(mocker: MockerFixture):
    # given
    bc_integration.bc_api_key = "abcd1234-abcd-1234-abcd-1234abcd1234"
    scanner_mock = MagicMock()
    scanner_mock.return_value.scan.return_value = dict()
    mocker.patch("checkov.sca_package_2.runner.Scanner", side_effect=scanner_mock)

    # when
    report = Runner().run(root_folder=EXAMPLES_DIR)

    # then
    assert report.check_type == "sca_package"
    assert report.resources == set()


def test_run_with_ide_source_and_bc_api_key(mocker: MockerFixture):
    # given
    bc_integration.bc_api_key = "abcd1234-abcd-1234-abcd-1234abcd1234"
    bc_integration.bc_source = SourceTypes[BCSourceType.JETBRAINS]

    scanner_mock = MagicMock()
    mocker.patch("checkov.sca_package_2.runner.Scanner", side_effect=scanner_mock)

    # when
    report = Runner().run(root_folder=EXAMPLES_DIR)

    #
    bc_integration.bc_source = None

    # then
    assert report.check_type == "sca_package"
    assert report.resources == set()
    assert report.error_status == ErrorStatus.SUCCESS  # shouldn't be ERROR

    # scanner shouldn't be invoked
    scanner_mock.assert_not_called()


import unittest

from checkov.terraform import TFDefinitionKey
from checkov.terraform.tf_parser import TFParser
from checkov.terraform.context_parsers.registry import parser_registry
import os


class TestVariableContextParser(unittest.TestCase):
    def setUp(self):
        test_root_dir = os.path.dirname(os.path.realpath(__file__)) + '/../evaluation/resources/default_evaluation/'
        parsing_errors = {}
        tf_definitions = TFParser().parse_directory(directory=test_root_dir,
                                 out_evaluations_context={},
                                 out_parsing_errors=parsing_errors)
        for definition in tf_definitions.items():
            definitions_context = parser_registry.enrich_definitions_context(definition)
        self.definitions_context = definitions_context

    def test_assignments_exists(self):
        file_path = os.path.dirname(os.path.realpath(__file__))\
                    + '/../evaluation/resources/default_evaluation/variables.tf'
        key = TFDefinitionKey(file_path=file_path, tf_source_modules=None)
        self.assertIsNotNone(
            self.definitions_context[key][
                'variable'].get(
                'assignments'))

    def test_assignment_value(self):
        file_path = os.path.dirname(os.path.realpath(__file__)) + '/../evaluation/resources/default_evaluation/variables.tf'
        key = TFDefinitionKey(file_path=file_path, tf_source_modules=None)
        self.assertFalse(
            self.definitions_context[key][
                'variable'].get(
                'assignments').get('user_exists')
        )

        self.assertEqual(
            self.definitions_context[key][
                'variable'].get(
                'assignments').get('app_client_id'), 'Temp')


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from pathlib import Path
from checkov.cloudformation.context_parser import ContextParser
from checkov.cloudformation.runner import Runner
from checkov.cloudformation.parser import parse, cfn_yaml
from checkov.runner_filter import RunnerFilter
from checkov.serverless.parsers.parser import parse as serverless_parse


class TestCfnYaml(unittest.TestCase):

    def test_skip_parsing(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))

        test_files = f'{current_dir}/skip.yaml'
        report = Runner().run(None, files=[test_files], runner_filter=RunnerFilter())
        summary = report.get_summary()

        self.assertEqual(summary['passed'], 2)
        self.assertEqual(summary['failed'], 3)
        self.assertEqual(summary['skipped'], 1)
        self.assertEqual(summary['parsing_errors'], 0)

    def test_file_inclusion(self):
        file = Path(__file__).resolve().parent / 'cfn_file.yaml'
        data, _ = serverless_parse(file)
        assert isinstance(data['resources'], list)
        assert len(data['resources']) == 1
        assert isinstance(data['resources'][0], dict)
        assert 'Resources' in data['resources'][0]
        assert isinstance(data['resources'][0]['Resources'], dict)

    def test_file_circular_inclusion(self):
        file = Path(__file__).resolve().parent / 'cfn_file_circular.yaml'
        with self.assertRaises(cfn_yaml.CfnParseError):
            cfn_yaml.load(file, cfn_yaml.ContentType.SLS)

    def test_code_line_extraction(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))

        # the test data that we'll evaluate against
        # line ranges are 1-based
        # mapping is file name, to resource index, to resource details
        # checking the resource index helps make sure that we are testing what we think we are testing
        files = [f'{current_dir}/cfn_newline_at_end.yaml', f'{current_dir}/cfn_nonewline_at_end.yaml']
        resource_properties_mapping = {
            files[0]: {
                0: {
                    'name': 'MyDB',
                    'line_range': [2, 9]
                },
                1: {
                    'name': 'MyBucket',
                    'line_range': [10, 13]
                }
            },
            files[1]: {
                0: {
                    'name': 'MyDB',
                    'line_range': [2, 9]
                },
                1: {
                    'name': 'MyBucket',
                    'line_range': [11, 14]
                }
            }
        }

        for file in files:
            cfn_dict, cfn_str = parse(file)

            cf_context_parser = ContextParser(file, cfn_dict, cfn_str)

            for index, (resource_name, resource) in enumerate(cfn_dict['Resources'].items()):
                # this filters out __startline__ and __endline__ markers
                resource_id = cf_context_parser.extract_cf_resource_id(resource, resource_name)
                if resource_id:
                    # make sure we are checking the right resource
                    self.assertEqual(resource_name, resource_properties_mapping[file][index]['name'])

                    entity_lines_range, entity_code_lines = cf_context_parser.extract_cf_resource_code_lines(resource)
                    self.assertEqual(entity_lines_range[0], entity_code_lines[0][0])
                    self.assertEqual(entity_lines_range[1], entity_code_lines[-1][0])
                    self.assertEqual(entity_lines_range, resource_properties_mapping[file][index]['line_range'])

    def test_trim_lines(self):
        # trim from front
        test1 = [
            (0, '\n'),
            (1, ''),
            (2, ' here is text'),
            (3, 'more text')
        ]

        self.assertEqual(ContextParser.trim_lines(test1), test1[2:4])

        # trim from back
        test2 = [
            (0, ' here is text'),
            (1, 'more text'),
            (2, '\n'),
            (3, ''),
        ]

        self.assertEqual(ContextParser.trim_lines(test2), test2[0:2])

        # trim from both
        test3 = [
            (0, '\n'),
            (1, ''),
            (2, ' here is text'),
            (3, 'more text'),
            (4, '\n'),
            (5, ''),
        ]

        self.assertEqual(ContextParser.trim_lines(test3), test3[2:4])

        # trim nothing
        test4 = [
            (2, ' here is text'),
            (3, 'more text'),
        ]

        self.assertEqual(ContextParser.trim_lines(test4), test4)

        # trim everything
        test5 = [
            (2, ''),
            (3, '\n'),
        ]

        self.assertEqual(ContextParser.trim_lines(test5), [])

    def test_parameter_import_lines(self):
        # check that when a parameter is imported into a resource, the line numbers of the resource are preserved
        current_dir = os.path.dirname(os.path.realpath(__file__))
        file = f'{current_dir}/cfn_with_ref.yaml'
        definitions, definitions_raw = parse(file)

        cf_context_parser = ContextParser(file, definitions, definitions_raw)
        resource = definitions['Resources']['ElasticsearchDomain']
        entity_lines_range, entity_code_lines = cf_context_parser.extract_cf_resource_code_lines(resource)
        self.assertEqual(entity_lines_range[0], 10)
        self.assertEqual(entity_lines_range[1], 20)

    def test_parsing_error(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))

        test_files = ["cfn_bad_name.yaml", "cfn_with_ref_bad.yaml", "cfn_bad_iam.yaml"]
        report = Runner().run(None, files=[f'{current_dir}/{f}' for f in test_files], runner_filter=RunnerFilter())
        summary = report.get_summary()

        self.assertEqual(summary['passed'], 6)
        self.assertEqual(summary['failed'], 0)
        self.assertEqual(summary['skipped'], 0)
        self.assertEqual(summary['parsing_errors'], 2)


if __name__ == '__main__':
    unittest.main()



from pathlib import Path

from checkov.runner_filter import RunnerFilter
from checkov.secrets.runner import Runner


def test_multiline_keyword_password_in_pod():
    # given
    test_file_path = Path(__file__).parent / "terraform_multiline/pod.tf"

    #  when
    report = Runner().run(
        root_folder=None, files=[str(test_file_path)], runner_filter=RunnerFilter(framework=["secrets"])
    )

    #  then
    failing_resources = {
        "dcbf46de362e1b6942054b89ee293984e9a8a40a",
        "ac236b0474a9a702f99dbe244a14548783ace5c5",
        "9ed4f1457a9c27dd868c1f21276c6d7098d2bacf",
        "06af723e58378574456be0b4c41a89194aaed0c3",
        "5db2fafebcfed9b4c9ffc570c46ef2ca94a3881a",
    }

    failed_check_resources = {c.resource for c in report.failed_checks}

    assert len(report.passed_checks) == 0
    assert len(report.failed_checks) == len(failing_resources)
    assert len(report.skipped_checks) == 0
    assert len(report.parsing_errors) == 0

    assert failing_resources == failed_check_resources


def test_multiline_keyword_password_in_jsonencode():
    # given
    test_file_path = Path(__file__).parent / "terraform_multiline/ecs_jsonencode.tf"

    #  when
    report = Runner().run(
        root_folder=None, files=[str(test_file_path)], runner_filter=RunnerFilter(framework=["secrets"])
    )

    #  then
    failing_resources = {
        "dcbf46de362e1b6942054b89ee293984e9a8a40a",
        "ac236b0474a9a702f99dbe244a14548783ace5c5",
        "9ed4f1457a9c27dd868c1f21276c6d7098d2bacf",
        "06af723e58378574456be0b4c41a89194aaed0c3",
        "5db2fafebcfed9b4c9ffc570c46ef2ca94a3881a",
    }

    failed_check_resources = {c.resource for c in report.failed_checks}

    assert len(report.passed_checks) == 0
    assert len(report.failed_checks) == len(failing_resources)
    assert len(report.skipped_checks) == 0
    assert len(report.parsing_errors) == 0

    assert failing_resources == failed_check_resources


def test_multiline_keyword_password_in_json_heredoc():
    # given
    test_file_path = Path(__file__).parent / "terraform_multiline/ecs_heredoc.tf"

    #  when
    report = Runner().run(
        root_folder=None, files=[str(test_file_path)], runner_filter=RunnerFilter(framework=["secrets"])
    )

    #  then
    failing_resources = {
        "dcbf46de362e1b6942054b89ee293984e9a8a40a",
        "ac236b0474a9a702f99dbe244a14548783ace5c5",
        "9ed4f1457a9c27dd868c1f21276c6d7098d2bacf",
        "06af723e58378574456be0b4c41a89194aaed0c3",
        "5db2fafebcfed9b4c9ffc570c46ef2ca94a3881a",
    }

    failed_check_resources = {c.resource for c in report.failed_checks}

    assert len(report.passed_checks) == 0
    assert len(report.failed_checks) == len(failing_resources)
    assert len(report.skipped_checks) == 0
    assert len(report.parsing_errors) == 0

    assert failing_resources == failed_check_resources


def test_multiline_keyword_password_in_yaml_heredoc():
    # given
    test_file_path = Path(__file__).parent / "terraform_multiline/cfn_heredoc.tf"

    #  when
    report = Runner().run(
        root_folder=None, files=[str(test_file_path)], runner_filter=RunnerFilter(framework=["secrets"])
    )

    #  then
    failing_resources = {
        "dcbf46de362e1b6942054b89ee293984e9a8a40a",
        "ac236b0474a9a702f99dbe244a14548783ace5c5",
        "9ed4f1457a9c27dd868c1f21276c6d7098d2bacf",
        "06af723e58378574456be0b4c41a89194aaed0c3",
        "5db2fafebcfed9b4c9ffc570c46ef2ca94a3881a",
    }

    failed_check_resources = {c.resource for c in report.failed_checks}

    assert len(report.passed_checks) == 0
    assert len(report.failed_checks) == len(failing_resources)
    assert len(report.skipped_checks) == 0
    assert len(report.parsing_errors) == 0

    assert failing_resources == failed_check_resources


def test_multiline_keyword_password_skip_data_blocks():
    # given
    test_file_path = Path(__file__).parent / "terraform_multiline/data.tf"

    #  when
    report = Runner().run(
        root_folder=None, files=[str(test_file_path)], runner_filter=RunnerFilter(framework=["secrets"])
    )

    #  then
    assert len(report.passed_checks) == 0
    assert len(report.failed_checks) == 0
    assert len(report.skipped_checks) == 0
    assert len(report.parsing_errors) == 0



import os
import unittest
from typing import List

from unittest import mock
from parameterized import parameterized

from checkov.common.checks.base_check import BaseCheck
from checkov.common.checks.base_check_registry import BaseCheckRegistry
from checkov.common.models.enums import CheckResult


class TestCheckTypeNotInSignature(BaseCheck):
    # for pytest not to collect this class as tests
    __test__ = False

    def __init__(self):
        name = "Example check"
        categories = []
        id = "CKV_T_1"
        supported_entities = ["module"]
        block_type = "module"
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_entities,
                         block_type=block_type)

    # noinspection PyMethodOverriding
    def scan_entity_conf(self, conf, entity_type):
        """
        My documentation
        :param conf:
        :return:
        """
        return CheckResult.PASSED


class TestCheckDetails(BaseCheck):
    # for pytest not to collect this class as tests
    __test__ = False

    def __init__(self, fail_check=False):
        name = "Another Example check"
        categories = []
        id = "CKV_T_2"
        supported_entities = ["my_resource_type"]
        block_type = "resource"
        self.fail_check = fail_check
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_entities,
                         block_type=block_type)

    # noinspection PyMethodOverriding
    def scan_entity_conf(self, conf, entity_type):
        """
        My documentation
        :param conf:
        :return:
        """
        if self.fail_check:
            raise Exception("An error")
        if conf.get("value")[0]:
            self.details.append("This check PASSED...")
            return CheckResult.PASSED
        else:
            self.details.append("This check FAILED...")
            return CheckResult.FAILED


def _clean_doc(st: str) -> List[str]:
    return [line.strip() for line in st.splitlines() if not line.isspace()]


# noinspection DuplicatedCode
class TestBaseCheck(unittest.TestCase):

    def test_entity_type_is_not_required_in_signature(self):
        registry = BaseCheckRegistry(report_type='')
        check = TestCheckTypeNotInSignature()
        registry.register(check)

        # noinspection PyArgumentList
        scan_result = check.scan_entity_conf({}, "Some name")
        self.assertEqual(CheckResult.PASSED, scan_result)
        self.assertEqual(_clean_doc(check.scan_entity_conf.__doc__), _clean_doc("""
        My documentation
        :param conf:
        :return:
        """))

    def test_invalid_signature_is_detected(self):
        with self.assertRaises(TypeError) as context:
            class TestCheckUnknownSignature(BaseCheck):

                def __init__(self):
                    name = "Example check"
                    categories = []
                    id = "CKV_T_1"
                    supported_entities = ["module"]
                    block_type = "module"
                    super().__init__(name=name, id=id, categories=categories, supported_entities=supported_entities,
                                     block_type=block_type)

            TestCheckUnknownSignature()

        self.assertIsInstance(context.exception, TypeError)
        self.assertRegex(context.exception.args[0], r"Can't instantiate abstract class TestCheckUnknownSignature")

    def test_details_reinitializing_after_execution(self):
        check = TestCheckDetails()
        self.assertEqual(0, len(check.details))
        result = check.run("test.tf", {"value": ["True"]}, "my_resource", "resource", {})
        self.assertEqual(CheckResult.PASSED, result["result"])
        self.assertEqual(1, len(check.details))
        self.assertIn("This check PASSED...", check.details)
        result = check.run("test.tf", {"value": [""]}, "my_resource_2", "resource", {})
        self.assertEqual(CheckResult.FAILED, result["result"])
        self.assertEqual(1, len(check.details))
        self.assertIn("This check FAILED...", check.details)

    @parameterized.expand([
        ("WARNING",),
        ("ERROR",)
    ])
    def test_check_fail_log_level_error(self, log_level):
        with self.assertLogs(level=log_level) as log, mock.patch.dict(os.environ,
                                                                      {'CHECKOV_CHECK_FAIL_LEVEL': log_level}, clear=True):
            check = TestCheckDetails(fail_check=True)
            self.assertEqual(0, len(check.details))
            try:
                check.run("test.tf", {"value": ["True"]}, "my_resource", "resource", {})
            except Exception:
                self.assertEqual(len(log.output), 1)


if __name__ == '__main__':
    unittest.main()



from aws_cdk import core
from aws_cdk import aws_elasticache as elasticache

class ElastiCacheReplicationGroupStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create an Amazon ElastiCache replication group
        replication_group = elasticache.CfnReplicationGroup(
            self,
            "MyElastiCacheReplicationGroup",
            replication_group_description="My Replication Group",
            automatic_failover_enabled=True,
            replication_group_id="my-replication-group",
            cache_node_type="cache.m4.large",
            engine="redis",
            engine_version="5.0.6",
            num_node_groups=2,
            cache_subnet_group_name="my-subnet-group",
            security_group_ids=["sg-0123456789abcdef0"],
        )

app = core.App()
ElastiCacheReplicationGroupStack(app, "ElastiCacheReplicationGroupStack")
app.synth()



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.MariaDBPublicAccessDisabled import check
from checkov.common.models.enums import CheckResult


class TestMariaDBPublicAccessDisabled(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_mariadb_server" "example" {
            name                = var.server_name
            location            = var.resource_group.location
            resource_group_name = var.resource_group.name
            administrator_login          = var.admin_login
            administrator_login_password = random_string.password.result
            sku_name   = "B_Gen5_2"
            storage_mb = 5120
            version    = "10.2"
            auto_grow_enabled             = true
            backup_retention_days         = 7
            geo_redundant_backup_enabled  = false
            public_network_access_enabled = true
            #test this i guess
            ssl_enforcement_enabled = false
        }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_mariadb_server']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
        resource "azurerm_mariadb_server" "example" {
            name                = var.server_name
            location            = var.resource_group.location
            resource_group_name = var.resource_group.name
            administrator_login          = var.admin_login
            administrator_login_password = random_string.password.result
            sku_name   = "B_Gen5_2"
            storage_mb = 5120
            version    = "10.2"
            auto_grow_enabled             = true
            backup_retention_days         = 7
            geo_redundant_backup_enabled  = false
            public_network_access_enabled = false
            #test this i guess
            ssl_enforcement_enabled = true
        }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_mariadb_server']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()
